@@ -23,7 +23,7 @@
  */
 
 /*
- * DOS/DJGPP device driver v1.0 for Mesa 4.0
+ * DOS/DJGPP device driver v1.1 for Mesa 4.0
  *
  *  Copyright (C) 2002 - Borca Daniel
  *  Email : dborca@yahoo.com
@@ -31,9 +31,6 @@
  */
 
 
-#ifdef PC_HEADER
-#include "all.h"
-#else
 #include "glheader.h"
 #include "context.h"
 #include "GL/dmesa.h"
@@ -54,7 +51,6 @@
 #include "tnl/tnl.h"
 #include "tnl/t_context.h"
 #include "tnl/t_pipeline.h"
-#endif
 
 #include "video.h"
 
@@ -79,10 +75,9 @@ struct dmesa_buffer {
    GLframebuffer gl_buffer;     /* The depth, stencil, accum, etc buffers */
    void *the_window;            /* your window handle, etc */
 
-   int bypp;                    /* bytes per pixel */
    int xpos, ypos;              /* position */
    int width, height;           /* size in pixels */
-   int bwidth, len;             /* bytes in a line, then total */
+   int bypp, stride, bytes;     /* bytes per pixel, in a line, then total */
 };
 
 /*
@@ -412,7 +407,7 @@ static swrast_tri_func dmesa_choose_tri_function (GLcontext *ctx)
 
  if (ctx->RenderMode != GL_RENDER)  return (swrast_tri_func) NULL;
  if (ctx->Polygon.SmoothFlag)       return (swrast_tri_func) NULL;
- if (ctx->Texture._ReallyEnabled)   return (swrast_tri_func) NULL;
+ if (ctx->Texture._EnabledUnits)    return (swrast_tri_func) NULL;
 
  if (ctx->Light.ShadeModel==GL_SMOOTH
      && swrast->_RasterMask==DEPTH_BIT
@@ -468,8 +463,13 @@ static void dmesa_choose_tri (GLcontext *ctx)
 
 static void clear_color (GLcontext *ctx, const GLchan color[4])
 {
+ const GLubyte col[4];
  DMesaContext c = (DMesaContext)ctx->DriverCtx;
- c->ClearColor = vl_mixrgba(color);
+ CLAMPED_FLOAT_TO_UBYTE(col[0], color[0]);
+ CLAMPED_FLOAT_TO_UBYTE(col[1], color[1]);
+ CLAMPED_FLOAT_TO_UBYTE(col[2], color[2]);
+ CLAMPED_FLOAT_TO_UBYTE(col[3], color[3]);
+ c->ClearColor = vl_mixrgba(col);
 }
 
 
@@ -493,7 +493,7 @@ static void clear (GLcontext *ctx, GLbitfield mask, GLboolean all,
  if (*colorMask==0xffffffff) {
     if (mask & DD_BACK_LEFT_BIT) {
        if (all) {
-          vl_clear(b->the_window, b->len, c->ClearColor);
+          vl_clear(b->the_window, b->bytes, c->ClearColor);
        } else {
           vl_rect(b->the_window, x, y, width, height, c->ClearColor);
        }
@@ -514,10 +514,9 @@ static void clear (GLcontext *ctx, GLbitfield mask, GLboolean all,
 static void set_read_buffer (GLcontext *ctx, GLframebuffer *buffer,
                              GLenum mode)
 {
-/*
- DMesaContext c = (DMesaContext)ctx->DriverCtx;
- dmesa_update_state(ctx);
-*/
+   /*
+     XXX this has to be fixed
+   */
 }
 
 
@@ -553,7 +552,7 @@ static const GLubyte* get_string (GLcontext *ctx, GLenum name)
 {
  switch (name) {
         case GL_RENDERER:
-             return (const GLubyte *)"Mesa DOS\0DJGPP port (c) Borca Daniel 31-mar-2002";
+             return (const GLubyte *)"Mesa DJGPP\0port (c) Borca Daniel 3-sep-2002";
         default:
              return NULL;
  }
@@ -651,16 +650,19 @@ void dmesa_init_pointers (GLcontext *ctx)
  ctx->Driver.TexSubImage3D = _mesa_store_texsubimage3d;
  ctx->Driver.TestProxyTexImage = _mesa_test_proxy_teximage;
 
+ ctx->Driver.CompressedTexImage1D = _mesa_store_compressed_teximage1d;
+ ctx->Driver.CompressedTexImage2D = _mesa_store_compressed_teximage2d;
+ ctx->Driver.CompressedTexImage3D = _mesa_store_compressed_teximage3d;
+ ctx->Driver.CompressedTexSubImage1D = _mesa_store_compressed_texsubimage1d;
+ ctx->Driver.CompressedTexSubImage2D = _mesa_store_compressed_texsubimage2d;
+ ctx->Driver.CompressedTexSubImage3D = _mesa_store_compressed_texsubimage3d;
+
  ctx->Driver.CopyTexImage1D = _swrast_copy_teximage1d;
  ctx->Driver.CopyTexImage2D = _swrast_copy_teximage2d;
  ctx->Driver.CopyTexSubImage1D = _swrast_copy_texsubimage1d;
  ctx->Driver.CopyTexSubImage2D = _swrast_copy_texsubimage2d;
  ctx->Driver.CopyTexSubImage3D = _swrast_copy_texsubimage3d;
 
- ctx->Driver.BaseCompressedTexFormat = _mesa_base_compressed_texformat;
- ctx->Driver.CompressedTextureSize = _mesa_compressed_texture_size;
- ctx->Driver.GetCompressedTexImage = _mesa_get_compressed_teximage;
-
  /* Swrast hooks for imaging extensions:
   */
  ctx->Driver.CopyColorTable = _swrast_CopyColorTable;
@@ -729,6 +731,12 @@ DMesaVisual DMesaCreateVisual (GLint width, GLint height, GLint colDepth,
  DMesaVisual v;
  GLint redBits, greenBits, blueBits, alphaBits;
 
+ int refresh;
+ char *var = getenv("DMESA_REFRESH");
+ if ((var == NULL) || ((refresh=atoi(var)) == 0)) {
+    refresh = 60;
+ }
+
  if (!dbFlag) {
     return NULL;
  }
@@ -755,7 +763,7 @@ DMesaVisual DMesaCreateVisual (GLint width, GLint height, GLint colDepth,
              return NULL;
  }
 
- if (vl_video_init(width, height, colDepth)!=0) {
+ if (vl_video_init(width, height, colDepth, refresh) != 0) {
     return NULL;
  }
 
@@ -788,7 +796,7 @@ DMesaVisual DMesaCreateVisual (GLint width, GLint height, GLint colDepth,
 
 void DMesaDestroyVisual (DMesaVisual v)
 {
- vl_video_exit(!0);
+ vl_video_exit();
  _mesa_destroy_visual(v->gl_visual);
  free(v);
 }
@@ -884,8 +892,8 @@ GLboolean DMesaViewport (DMesaBuffer b,
     b->ypos = ypos;
     b->width = width;
     b->height = height;
-    b->bwidth = width * b->bypp;
-    b->len = b->bwidth * height;
+    b->stride = width * b->bypp;
+    b->bytes = b->stride * height;
     return GL_TRUE;
  }
 }
@@ -923,5 +931,7 @@ GLboolean DMesaMakeCurrent (DMesaContext c, DMesaBuffer b)
 void DMesaSwapBuffers (DMesaBuffer b)
 {
  /* copy/swap back buffer to front if applicable */
- vl_flip(b->the_window, b->bwidth, b->height);
+ GET_CURRENT_CONTEXT(ctx);
+ _mesa_notifySwapBuffers(ctx);
+ vl_flip(b->the_window, b->stride, b->height);
 }
