@@ -1,10 +1,10 @@
-/* $Id: fxdd.c,v 1.84.2.3 2002/06/24 17:37:53 brianp Exp $ */
+/* $Id: fxdd.c,v 1.93 2002/10/29 15:03:16 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.0.3
+ * Version:  4.0
  *
- * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -47,7 +47,6 @@
 #include "extensions.h"
 #include "texstore.h"
 #include "swrast/swrast.h"
-#include "swrast/s_context.h"
 #include "swrast_setup/swrast_setup.h"
 #include "tnl/tnl.h"
 #include "tnl/t_context.h"
@@ -126,17 +125,21 @@ fxDDBufferSize(GLframebuffer *buffer, GLuint * width, GLuint * height)
 
 /* Implements glClearColor() */
 static void
-fxDDClearColor(GLcontext * ctx, const GLchan color[4])
+fxDDClearColor(GLcontext * ctx, const GLfloat color[4])
 {
    fxMesaContext fxMesa = (fxMesaContext) ctx->DriverCtx;
    GLubyte col[4];
 
    if (MESA_VERBOSE & VERBOSE_DRIVER) {
-      fprintf(stderr, "fxmesa: fxDDClearColor(%d,%d,%d,%d)\n",
+      fprintf(stderr, "fxmesa: fxDDClearColor(%f,%f,%f,%f)\n",
 	      color[0], color[1], color[2], color[3]);
    }
 
-   ASSIGN_4V(col, color[0], color[1], color[2], 255);
+   CLAMPED_FLOAT_TO_UBYTE(col[0], color[0]);
+   CLAMPED_FLOAT_TO_UBYTE(col[1], color[1]);
+   CLAMPED_FLOAT_TO_UBYTE(col[2], color[2]);
+   CLAMPED_FLOAT_TO_UBYTE(col[3], color[3]);
+
    fxMesa->clearC = FXCOLOR4(col);
    fxMesa->clearA = color[3];
 }
@@ -246,7 +249,7 @@ fxDDClear(GLcontext * ctx, GLbitfield mask, GLboolean all,
       FX_grBufferClear(fxMesa->clearC, fxMesa->clearA, clearD);
       FX_grColorMask(FXTRUE, ctx->Color.ColorMask[ACOMP]
 		     && fxMesa->haveAlphaBuffer);
-      if (ctx->Color.DrawDestMask & FRONT_LEFT_BIT)
+      if (ctx->Color._DrawDestMask & FRONT_LEFT_BIT)
 	 FX_grRenderBuffer(GR_BUFFER_FRONTBUFFER);
       if (!ctx->Depth.Test || !ctx->Depth.Mask)
 	 FX_grDepthMask(FXFALSE);
@@ -289,10 +292,15 @@ fxDDSetDrawBuffer(GLcontext * ctx, GLenum mode)
       /* we'll need a software fallback */
       /* XXX not implemented */
    }
+
+   /* update s/w fallback state */
+   _swrast_DrawBuffer(ctx, mode);
 }
 
 
 
+
+
 static void
 fxDDDrawBitmap(GLcontext * ctx, GLint px, GLint py,
 	       GLsizei width, GLsizei height,
@@ -304,7 +312,6 @@ fxDDDrawBitmap(GLcontext * ctx, GLint px, GLint py,
    FxU16 color;
    const struct gl_pixelstore_attrib *finalUnpack;
    struct gl_pixelstore_attrib scissoredUnpack;
-   SWcontext *swrast = SWRAST_CONTEXT(ctx);
 
    /* check if there's any raster operations enabled which we can't handle */
    if (ctx->Color.AlphaEnabled ||
@@ -315,11 +322,14 @@ fxDDDrawBitmap(GLcontext * ctx, GLint px, GLint py,
        ctx->Stencil.Enabled ||
        ctx->Scissor.Enabled ||
        (ctx->DrawBuffer->UseSoftwareAlphaBuffers &&
-	ctx->Color.ColorMask[ACOMP]) || (swrast->_RasterMask & MULTI_DRAW_BIT)) {
+	ctx->Color.ColorMask[ACOMP]) ||
+       (ctx->Color._DrawDestMask != FRONT_LEFT_BIT &&
+        ctx->Color._DrawDestMask != BACK_LEFT_BIT)) {
       _swrast_Bitmap(ctx, px, py, width, height, unpack, bitmap);
       return;
    }
 
+
    if (ctx->Scissor.Enabled) {
       /* This is a bit tricky, but by carefully adjusting the px, py,
        * width, height, skipPixels and skipRows values we can do
@@ -711,7 +721,7 @@ fxDDInitFxMesaContext(fxMesaContext fxMesa)
    }
 
    fxMesa->state = malloc(FX_grGetInteger(FX_GLIDE_STATE_SIZE));
-   fxMesa->fogTable = malloc(FX_grGetInteger(FX_FOG_TABLE_ENTRIES) *
+   fxMesa->fogTable = (GrFog_t *) malloc(FX_grGetInteger(FX_FOG_TABLE_ENTRIES) *
 			     sizeof(GrFog_t));
 
    if (!fxMesa->state || !fxMesa->fogTable) {
@@ -763,9 +773,6 @@ fxDDInitFxMesaContext(fxMesaContext fxMesa)
 
    FX_grGlideGetState((GrState *) fxMesa->state);
 
-   /* Run the config file */
-   _mesa_read_config_file(fxMesa->glCtx);
-
    return 1;
 }
 
@@ -821,13 +828,13 @@ GLboolean
 fx_check_IsInHardware(GLcontext * ctx)
 {
    fxMesaContext fxMesa = (fxMesaContext) ctx->DriverCtx;
-   SWcontext *swrast = SWRAST_CONTEXT(ctx);
 
    if (ctx->RenderMode != GL_RENDER)
       return GL_FALSE;
 
    if (ctx->Stencil.Enabled ||
-       (swrast->_RasterMask & MULTI_DRAW_BIT) ||
+       (ctx->Color._DrawDestMask != FRONT_LEFT_BIT &&
+        ctx->Color._DrawDestMask != BACK_LEFT_BIT) ||
        ((ctx->Color.BlendEnabled)
 	&& (ctx->Color.BlendEquation != GL_FUNC_ADD_EXT))
        || ((ctx->Color.ColorLogicOpEnabled)
@@ -844,14 +851,15 @@ fx_check_IsInHardware(GLcontext * ctx)
    /* Unsupported texture/multitexture cases */
 
    if (fxMesa->haveTwoTMUs) {
-      if (ctx->Texture._ReallyEnabled & (TEXTURE0_3D | TEXTURE1_3D))
-	 return GL_FALSE;	/* can't do 3D textures */
-      if (ctx->Texture._ReallyEnabled & (TEXTURE0_1D | TEXTURE1_1D))
-	 return GL_FALSE;	/* can't do 1D textures */
+      /* we can only do 2D textures */
+      if (ctx->Texture.Unit[0]._ReallyEnabled & ~TEXTURE_2D_BIT)
+	 return GL_FALSE;
+      if (ctx->Texture.Unit[1]._ReallyEnabled & ~TEXTURE_2D_BIT)
+	 return GL_FALSE;
 
-      if (ctx->Texture._ReallyEnabled & TEXTURE0_2D) {
+      if (ctx->Texture.Unit[0]._ReallyEnabled & TEXTURE_2D_BIT) {
 	 if (ctx->Texture.Unit[0].EnvMode == GL_BLEND &&
-	     (ctx->Texture._ReallyEnabled & TEXTURE1_2D ||
+	     (ctx->Texture.Unit[1]._ReallyEnabled & TEXTURE_2D_BIT ||
 	      ctx->Texture.Unit[0].EnvColor[0] != 0 ||
 	      ctx->Texture.Unit[0].EnvColor[1] != 0 ||
 	      ctx->Texture.Unit[0].EnvColor[2] != 0 ||
@@ -862,7 +870,7 @@ fx_check_IsInHardware(GLcontext * ctx)
 	    return GL_FALSE;
       }
 
-      if (ctx->Texture._ReallyEnabled & TEXTURE1_2D) {
+      if (ctx->Texture.Unit[1]._ReallyEnabled & TEXTURE_2D_BIT) {
 	 if (ctx->Texture.Unit[1].EnvMode == GL_BLEND)
 	    return GL_FALSE;
 	 if (ctx->Texture.Unit[1]._Current->Image[0]->Border > 0)
@@ -876,9 +884,10 @@ fx_check_IsInHardware(GLcontext * ctx)
 
       /* KW: This was wrong (I think) and I changed it... which doesn't mean
        * it is now correct...
+       * BP: The old condition just seemed to test if both texture units
+       * were enabled.  That's easy!
        */
-      if ((ctx->Texture._ReallyEnabled & (TEXTURE0_1D | TEXTURE0_2D | TEXTURE0_3D)) &&
-	  (ctx->Texture._ReallyEnabled & (TEXTURE1_1D | TEXTURE1_2D | TEXTURE1_3D))) {
+      if (ctx->Texture._EnabledUnits == 0x3) {
 	 /* Can't use multipass to blend a multitextured triangle - fall
 	  * back to software.
 	  */
@@ -896,16 +905,12 @@ fx_check_IsInHardware(GLcontext * ctx)
       }
    }
    else {
-      if ((ctx->Texture._ReallyEnabled & (TEXTURE1_1D | TEXTURE1_2D | TEXTURE1_3D)) ||
-	  /* Not very well written ... */
-	  ((ctx->Texture._ReallyEnabled & TEXTURE0_1D) &&
-           (!(ctx->Texture._ReallyEnabled & TEXTURE0_2D)))
-	 ) {
+      /* we have just one texture unit */
+      if (ctx->Texture._EnabledUnits > 0x1) {
 	 return GL_FALSE;
       }
 
-
-      if ((ctx->Texture._ReallyEnabled & TEXTURE0_2D) &&
+      if ((ctx->Texture.Unit[0]._ReallyEnabled & TEXTURE_2D_BIT) &&
 	  (ctx->Texture.Unit[0].EnvMode == GL_BLEND)) {
 	 return GL_FALSE;
       }
@@ -996,7 +1001,7 @@ fxSetupDDPointers(GLcontext * ctx)
    ctx->Driver.ClearIndex = NULL;
    ctx->Driver.ClearColor = fxDDClearColor;
    ctx->Driver.Clear = fxDDClear;
-   ctx->Driver.SetDrawBuffer = fxDDSetDrawBuffer;
+   ctx->Driver.DrawBuffer = fxDDSetDrawBuffer;
    ctx->Driver.GetBufferSize = fxDDBufferSize;
    ctx->Driver.Accum = _swrast_Accum;
    ctx->Driver.Bitmap = fxDDDrawBitmap;
@@ -1013,6 +1018,12 @@ fxSetupDDPointers(GLcontext * ctx)
    ctx->Driver.TexSubImage1D = _mesa_store_texsubimage1d;
    ctx->Driver.TexSubImage2D = fxDDTexSubImage2D;
    ctx->Driver.TexSubImage3D = _mesa_store_texsubimage3d;
+   ctx->Driver.CompressedTexImage1D = _mesa_store_compressed_teximage1d;
+   ctx->Driver.CompressedTexImage2D = _mesa_store_compressed_teximage2d;
+   ctx->Driver.CompressedTexImage3D = _mesa_store_compressed_teximage3d;
+   ctx->Driver.CompressedTexSubImage1D = _mesa_store_compressed_texsubimage1d;
+   ctx->Driver.CompressedTexSubImage2D = _mesa_store_compressed_texsubimage2d;
+   ctx->Driver.CompressedTexSubImage3D = _mesa_store_compressed_texsubimage3d;
    ctx->Driver.CopyTexImage1D = _swrast_copy_teximage1d;
    ctx->Driver.CopyTexImage2D = _swrast_copy_teximage2d;
    ctx->Driver.CopyTexSubImage1D = _swrast_copy_texsubimage1d;
