@@ -1,4 +1,4 @@
-/* $Id: fxtris.c,v 1.17.2.1 2001/11/06 16:00:58 brianp Exp $ */
+/* $Id: fxtris.c,v 1.20 2002/10/24 23:57:23 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -28,16 +28,10 @@
  *    Keith Whitwell <keithw@valinux.com>
  */
 
-#ifdef HAVE_CONFIG_H
-#include "conf.h"
-#endif
+#include "glheader.h"
 
 #ifdef FX
 
-#include <stdio.h>
-#include <math.h>
-
-#include "glheader.h"
 #include "mtypes.h"
 #include "macros.h"
 #include "colormac.h"
@@ -331,7 +325,7 @@ do {						\
 do {						\
    color[idx][0] = v[idx]->r;			\
    color[idx][1] = v[idx]->g;			\
-   color[idx][1] = v[idx]->b;			\
+   color[idx][2] = v[idx]->b;			\
    color[idx][3] = v[idx]->a;			\
 } while (0)
 
