@@ -1,4 +1,4 @@
-/* $Id: wmesa.c,v 1.22.2.8 2002/06/24 20:18:04 kschultz Exp $ */
+/* $Id: wmesa.c,v 1.41 2002/10/24 23:57:23 brianp Exp $ */
 
 /*
  * Windows (Win32) device driver for Mesa 3.4
@@ -36,9 +36,9 @@
 #include "dd.h"
 #include "depth.h"
 #include "extensions.h"
+#include "imports.h"
 #include "macros.h"
 #include "matrix.h"
-#include "mem.h"
 #include "mmath.h"
 #include "mtypes.h"
 #include "texformat.h"
@@ -100,7 +100,13 @@ WMesaContext WC = NULL;
 
 #ifdef COMPILE_SETPIXEL
 
-__forceinline void wmSetPixel(PWMC pwc, int iScanLine, int iPixel, BYTE r, BYTE g, BYTE b)
+#if defined(_MSC_VER) && _MSC_VER >= 1200
+#define FORCEINLINE __forceinline
+#else
+#define FORCEINLINE __inline
+#endif
+
+FORCEINLINE void wmSetPixel(PWMC pwc, int iScanLine, int iPixel, BYTE r, BYTE g, BYTE b)
 {
 	pwc->wmSetPixel(pwc,iScanLine,iPixel,r,g,b);
 }
@@ -364,9 +370,13 @@ static void clear_index(GLcontext* ctx, GLuint index)
 /*
  * Set the color used to clear the color buffer.
  */
-static void clear_color( GLcontext* ctx, const GLchan color[4] )
+static void clear_color( GLcontext* ctx, const GLfloat color[4] )
 {
-  Current->clearpixel = RGB(color[0], color[1], color[2]);
+  GLubyte col[4];
+  CLAMPED_FLOAT_TO_UBYTE(col[0], color[0]);
+  CLAMPED_FLOAT_TO_UBYTE(col[1], color[1]);
+  CLAMPED_FLOAT_TO_UBYTE(col[2], color[2]);
+  Current->clearpixel = RGB(col[0], col[1], col[2]);
 }
 
 
@@ -568,20 +578,12 @@ static void enable( GLcontext* ctx, GLenum pname, GLboolean enable )
   }
 }
 
-static void set_draw_buffer( GLcontext* ctx, GLenum mode )
-{
-   /* XXX doing nothing for now */
-   /* if front buffer, fine */
-   /* if back buffer, fine */
-   /* else, check swrast->_RasterMask & MULTI_DRAW_BIT, if true, */
-   /* use a swrast fallback function */
-}
 
 
-static void set_read_buffer(GLcontext *ctx, GLframebuffer *colorBuffer,
-                            GLenum buffer )
+static void set_buffer(GLcontext *ctx, GLframebuffer *colorBuffer,
+                       GLuint bufferBit )
 {
-  /* XXX todo */
+  /* XXX todo - examine bufferBit and set read/write pointers */
   return;
 }
 
@@ -1026,7 +1028,6 @@ static void SetFunctionPointers(GLcontext *ctx)
   struct swrast_device_driver *swdd = _swrast_GetDeviceDriverReference( ctx );
   ctx->Driver.GetString = get_string;
   ctx->Driver.UpdateState = wmesa_update_state;
-  ctx->Driver.SetDrawBuffer = set_draw_buffer;
   ctx->Driver.ResizeBuffers = _swrast_alloc_buffers;
   ctx->Driver.GetBufferSize = buffer_size;
   
@@ -1042,6 +1043,7 @@ static void SetFunctionPointers(GLcontext *ctx)
   ctx->Driver.CopyPixels = _swrast_CopyPixels;
   ctx->Driver.DrawPixels = _swrast_DrawPixels;
   ctx->Driver.ReadPixels = _swrast_ReadPixels;
+  ctx->Driver.DrawBuffer = _swrast_DrawBuffer;
   
   ctx->Driver.ChooseTextureFormat = _mesa_choose_tex_format;
   ctx->Driver.TexImage1D = _mesa_store_teximage1d;
@@ -1052,6 +1054,13 @@ static void SetFunctionPointers(GLcontext *ctx)
   ctx->Driver.TexSubImage3D = _mesa_store_texsubimage3d;
   ctx->Driver.TestProxyTexImage = _mesa_test_proxy_teximage;
   
+  ctx->Driver.CompressedTexImage1D = _mesa_store_compressed_teximage1d;
+  ctx->Driver.CompressedTexImage2D = _mesa_store_compressed_teximage2d;
+  ctx->Driver.CompressedTexImage3D = _mesa_store_compressed_teximage3d;
+  ctx->Driver.CompressedTexSubImage1D = _mesa_store_compressed_texsubimage1d;
+  ctx->Driver.CompressedTexSubImage2D = _mesa_store_compressed_texsubimage2d;
+  ctx->Driver.CompressedTexSubImage3D = _mesa_store_compressed_texsubimage3d;
+
   ctx->Driver.CopyTexImage1D = _swrast_copy_teximage1d;
   ctx->Driver.CopyTexImage2D = _swrast_copy_teximage2d;
   ctx->Driver.CopyTexSubImage1D = _swrast_copy_texsubimage1d;
@@ -1061,15 +1070,9 @@ static void SetFunctionPointers(GLcontext *ctx)
   ctx->Driver.CopyColorSubTable = _swrast_CopyColorSubTable;
   ctx->Driver.CopyConvolutionFilter1D = _swrast_CopyConvolutionFilter1D;
   ctx->Driver.CopyConvolutionFilter2D = _swrast_CopyConvolutionFilter2D;
-  
-  ctx->Driver.BaseCompressedTexFormat = _mesa_base_compressed_texformat;
-  ctx->Driver.CompressedTextureSize = _mesa_compressed_texture_size;
-  ctx->Driver.GetCompressedTexImage = _mesa_get_compressed_teximage;
-  
-  
-  swdd->SetReadBuffer = set_read_buffer;
-  
-  
+
+  swdd->SetBuffer = set_buffer;
+
   /* Pixel/span writing functions: */
   swdd->WriteRGBASpan        = write_rgba_span;
   swdd->WriteRGBSpan         = write_rgb_span;
@@ -1105,7 +1108,6 @@ static void wmesa_update_state( GLcontext *ctx, GLuint new_state )
 #if 0
   ctx->Driver.GetString = get_string;
   ctx->Driver.UpdateState = wmesa_update_state;
-  ctx->Driver.SetDrawBuffer = set_draw_buffer;
   ctx->Driver.ResizeBuffers = _swrast_alloc_buffers;
   ctx->Driver.GetBufferSize = buffer_size;
   
@@ -1131,6 +1133,13 @@ static void wmesa_update_state( GLcontext *ctx, GLuint new_state )
   ctx->Driver.TexSubImage3D = _mesa_store_texsubimage3d;
   ctx->Driver.TestProxyTexImage = _mesa_test_proxy_teximage;
   
+  ctx->Driver.CompressedTexImage1D = _mesa_store_compressed_teximage1d;
+  ctx->Driver.CompressedTexImage2D = _mesa_store_compressed_teximage2d;
+  ctx->Driver.CompressedTexImage3D = _mesa_store_compressed_teximage3d;
+  ctx->Driver.CompressedTexSubImage1D = _mesa_store_compressed_texsubimage1d;
+  ctx->Driver.CompressedTexSubImage2D = _mesa_store_compressed_texsubimage2d;
+  ctx->Driver.CompressedTexSubImage3D = _mesa_store_compressed_texsubimage3d;
+
   ctx->Driver.CopyTexImage1D = _swrast_copy_teximage1d;
   ctx->Driver.CopyTexImage2D = _swrast_copy_teximage2d;
   ctx->Driver.CopyTexSubImage1D = _swrast_copy_texsubimage1d;
@@ -1141,13 +1150,7 @@ static void wmesa_update_state( GLcontext *ctx, GLuint new_state )
   ctx->Driver.CopyConvolutionFilter1D = _swrast_CopyConvolutionFilter1D;
   ctx->Driver.CopyConvolutionFilter2D = _swrast_CopyConvolutionFilter2D;
   
-  ctx->Driver.BaseCompressedTexFormat = _mesa_base_compressed_texformat;
-  ctx->Driver.CompressedTextureSize = _mesa_compressed_texture_size;
-  ctx->Driver.GetCompressedTexImage = _mesa_get_compressed_teximage;
-  
-  
-  swdd->SetReadBuffer = set_read_buffer;
-  
+  swdd->SetBuffer = set_buffer;
   
   /* Pixel/span writing functions: */
   swdd->WriteRGBASpan        = write_rgba_span;
@@ -1251,6 +1254,7 @@ WMesaContext WMesaCreateContext( HWND hWnd, HPALETTE* Pal,
   RECT CR;
   WMesaContext c;
   GLboolean true_color_flag;
+
   c = (struct wmesa_context * ) calloc(1,sizeof(struct wmesa_context));
   if (!c)
     return NULL;
@@ -1338,7 +1342,7 @@ WMesaContext WMesaCreateContext( HWND hWnd, HPALETTE* Pal,
   }
   
   /* allocate a new Mesa context */
-  c->gl_ctx = _mesa_create_context( c->gl_visual, NULL, c, GL_TRUE);
+  c->gl_ctx = _mesa_create_context( c->gl_visual, NULL, (void *) c, GL_FALSE );
   
   if (!c->gl_ctx) {
     _mesa_destroy_visual( c->gl_visual );
@@ -1346,16 +1350,6 @@ WMesaContext WMesaCreateContext( HWND hWnd, HPALETTE* Pal,
     return NULL;
   }
   
-  if (!_mesa_initialize_context(c->gl_ctx,
-				c->gl_visual,
-				(GLcontext *) NULL,
-				(void *) c, GL_TRUE )) {
-    _mesa_destroy_visual( c->gl_visual );
-    free(c);
-    return NULL;
-  }
-  
-  
   _mesa_enable_sw_extensions(c->gl_ctx);
   _mesa_enable_1_3_extensions(c->gl_ctx);
 
@@ -1407,6 +1401,7 @@ void WMesaDestroyContext( void )
   _mesa_destroy_visual( c->gl_visual );
   _mesa_destroy_framebuffer( c->gl_buffer );
   _mesa_free_context_data( c->gl_ctx );
+  free( (void *) c->gl_ctx);
     
   if (c->db_flag)
 #ifdef DDRAW
@@ -1419,6 +1414,9 @@ void WMesaDestroyContext( void )
   if(parallelMachine)
     PRDestroyRenderBuffer();
 #endif
+
+  // Destroyed context no longer valid
+  WMesaMakeCurrent( NULL );
 }
 
 
@@ -1457,7 +1455,7 @@ void WMesaSwapBuffers( void )
    * we have to flush any pending rendering commands first.
    */
   if (Current && Current->gl_ctx == ctx)
-    _mesa_swapbuffers(ctx);
+    _mesa_notifySwapBuffers(ctx);
   
   if (Current->db_flag)
     wmFlush(Current);
@@ -3196,7 +3194,7 @@ static triangle_func choose_triangle_function( GLcontext *ctx )
     int depth = wmesa->cColorBits;
 
     if (ctx->Polygon.SmoothFlag)     return NULL;
-    if (ctx->Texture._ReallyEnabled)  return NULL;
+    if (ctx->Texture._EnabledUnits)  return NULL;
     if (!wmesa->db_flag) return NULL;
     if (ctx->swrast->_RasterMask & MULTI_DRAW_BIT) return NULL;
 
