@@ -1,8 +1,8 @@
-/* $Id: xm_api.c,v 1.29.2.9 2002/06/17 23:32:43 brianp Exp $ */
+/* $Id: xm_api.c,v 1.47 2002/10/25 21:06:34 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.0.3
+ * Version:  4.0.2
  *
  * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
@@ -69,13 +69,10 @@
 #include "context.h"
 #include "extensions.h"
 #include "glthread.h"
+#include "imports.h"
 #include "matrix.h"
-#include "mem.h"
 #include "mmath.h"
 #include "mtypes.h"
-#ifdef HAVE_CONFIG_H
-#include "conf.h"
-#endif
 #include "macros.h"
 #include "texformat.h"
 #include "texstore.h"
@@ -170,7 +167,7 @@ static void error( const char *msg )
 {
    (void)DitherValues;		/* Muffle compiler */
 
-   if (getenv("MESA_DEBUG"))
+   if (_mesa_getenv("MESA_DEBUG"))
       fprintf( stderr, "X/Mesa error: %s\n", msg );
 }
 
@@ -269,7 +266,7 @@ static GLint gamma_adjust( GLfloat gamma, GLint value, GLint max )
    }
    else {
       double x = (double) value / (double) max;
-      return IROUND_POS((GLfloat) max * pow(x, 1.0F/gamma));
+      return IROUND_POS((GLfloat) max * _mesa_pow(x, 1.0F/gamma));
    }
 }
 
@@ -283,8 +280,6 @@ static GLint gamma_adjust( GLfloat gamma, GLint value, GLint max )
  *         visinfo - desribes the visual to be used for XImages
  * Return:  true number of bits per pixel for XImages
  */
-#define GET_BITS_PER_PIXEL(xmv) bits_per_pixel(xmv)
-
 #ifdef XFree86Server
 
 static int bits_per_pixel( XMesaVisual xmv )
@@ -509,7 +504,7 @@ static GLboolean alloc_shm_back_buffer( XMesaBuffer b )
    b->shminfo.shmid = shmget( IPC_PRIVATE, b->backimage->bytes_per_line
 			     * b->backimage->height, IPC_CREAT|0777 );
    if (b->shminfo.shmid < 0) {
-      if (getenv("MESA_DEBUG"))
+      if (_mesa_getenv("MESA_DEBUG"))
           perror("alloc_back_buffer");
       XDestroyImage( b->backimage );
       b->backimage = NULL;
@@ -521,7 +516,7 @@ static GLboolean alloc_shm_back_buffer( XMesaBuffer b )
    b->shminfo.shmaddr = b->backimage->data
                       = (char*)shmat( b->shminfo.shmid, 0, 0 );
    if (b->shminfo.shmaddr == (char *) -1) {
-      if (getenv("MESA_DEBUG"))
+      if (_mesa_getenv("MESA_DEBUG"))
           perror("alloc_back_buffer");
       XDestroyImage( b->backimage );
       shmctl( b->shminfo.shmid, IPC_RMID, 0 );
@@ -629,9 +624,7 @@ void xmesa_alloc_back_buffer( XMesaBuffer b )
 	 b->backimage = XMesaCreateImage(b->xm_visual->BitsPerPixel,
 					 b->width, b->height, NULL);
 #else
-      if (b->shm==0
-	  || alloc_shm_back_buffer(b)==GL_FALSE
-	  ) {
+      if (b->shm==0 || alloc_shm_back_buffer(b)==GL_FALSE) {
 	 /* Allocate a regular XImage for the back buffer. */
 	 b->backimage = XCreateImage( b->xm_visual->display,
                                       b->xm_visual->visinfo->visual,
@@ -879,7 +872,7 @@ static GLboolean setup_grayscale( int client, XMesaVisual v,
             buffer->pixel_to_b[xcol.pixel] = gray;
          }
 
-         if (colorsfailed && getenv("MESA_DEBUG")) {
+         if (colorsfailed && _mesa_getenv("MESA_DEBUG")) {
             fprintf( stderr,
                   "Note: %d out of 256 needed colors do not match exactly.\n",
                   colorsfailed );
@@ -959,7 +952,7 @@ static GLboolean setup_dithered_color( int client, XMesaVisual v,
             }
          }
 
-         if (colorsfailed && getenv("MESA_DEBUG")) {
+         if (colorsfailed && _mesa_getenv("MESA_DEBUG")) {
             fprintf( stderr,
                   "Note: %d out of %d needed colors do not match exactly.\n",
                   colorsfailed, _R*_G*_B );
@@ -993,19 +986,19 @@ static void setup_8bit_hpcr( XMesaVisual v )
 
    g = 1.0 / v->RedGamma;
    for (i=0; i<256; i++) {
-      GLint red = IROUND_POS(255.0 * pow( hpcr_rgbTbl[0][i]/255.0, g ));
+      GLint red = IROUND_POS(255.0 * _mesa_pow( hpcr_rgbTbl[0][i]/255.0, g ));
       v->hpcr_rgbTbl[0][i] = CLAMP( red, 16, 239 );
    }
 
    g = 1.0 / v->GreenGamma;
    for (i=0; i<256; i++) {
-      GLint green = IROUND_POS(255.0 * pow( hpcr_rgbTbl[1][i]/255.0, g ));
+      GLint green = IROUND_POS(255.0 * _mesa_pow( hpcr_rgbTbl[1][i]/255.0, g ));
       v->hpcr_rgbTbl[1][i] = CLAMP( green, 16, 239 );
    }
 
    g = 1.0 / v->BlueGamma;
    for (i=0; i<256; i++) {
-      GLint blue = IROUND_POS(255.0 * pow( hpcr_rgbTbl[2][i]/255.0, g ));
+      GLint blue = IROUND_POS(255.0 * _mesa_pow( hpcr_rgbTbl[2][i]/255.0, g ));
       v->hpcr_rgbTbl[2][i] = CLAMP( blue, 32, 223 );
    }
    v->undithered_pf = PF_HPCR;  /* can't really disable dithering for now */
@@ -1014,7 +1007,7 @@ static void setup_8bit_hpcr( XMesaVisual v )
    /* which method should I use to clear */
    /* GL_FALSE: keep the ordinary method  */
    /* GL_TRUE : clear with dither pattern */
-   v->hpcr_clear_flag = getenv("MESA_HPCR_CLEAR") ? GL_TRUE : GL_FALSE;
+   v->hpcr_clear_flag = _mesa_getenv("MESA_HPCR_CLEAR") ? GL_TRUE : GL_FALSE;
 
    if (v->hpcr_clear_flag) {
       v->hpcr_clear_pixmap = XMesaCreatePixmap(v->display,
@@ -1197,8 +1190,7 @@ static GLboolean initialize_visual_and_buffer( int client,
                                                XMesaBuffer b,
                                                GLboolean rgb_flag,
                                                XMesaDrawable window,
-                                               XMesaColormap cmap
-                                             )
+                                               XMesaColormap cmap )
 {
 #ifndef XFree86Server
    XGCValues gcvalues;
@@ -1209,7 +1201,7 @@ static GLboolean initialize_visual_and_buffer( int client,
    }
 
    /* Save true bits/pixel */
-   v->BitsPerPixel = GET_BITS_PER_PIXEL(v);
+   v->BitsPerPixel = bits_per_pixel(v);
    assert(v->BitsPerPixel > 0);
 
 
@@ -1250,7 +1242,7 @@ static GLboolean initialize_visual_and_buffer( int client,
       }
       v->index_bits = 0;
 
-      if (getenv("MESA_NO_DITHER")) {
+      if (_mesa_getenv("MESA_NO_DITHER")) {
 	 v->dithered_pf = v->undithered_pf;
       }
    }
@@ -1261,7 +1253,7 @@ static GLboolean initialize_visual_and_buffer( int client,
     * which can help Brian figure out what's going on when a user
     * reports bugs.
     */
-   if (getenv("MESA_INFO")) {
+   if (_mesa_getenv("MESA_INFO")) {
       fprintf(stderr, "X/Mesa visual = %p\n", (void *) v);
       fprintf(stderr, "X/Mesa dithered pf = %u\n", v->dithered_pf);
       fprintf(stderr, "X/Mesa undithered pf = %u\n", v->undithered_pf);
@@ -1479,7 +1471,7 @@ XMesaVisual XMesaCreateVisual( XMesaDisplay *display,
    GLint red_bits, green_bits, blue_bits, alpha_bits;
 
    /* For debugging only */
-   if (getenv("MESA_XSYNC")) {
+   if (_mesa_getenv("MESA_XSYNC")) {
       /* This makes debugging X easier.
        * In your debugger, set a breakpoint on _XError to stop when an
        * X protocol error is generated.
@@ -1537,7 +1529,7 @@ XMesaVisual XMesaCreateVisual( XMesaDisplay *display,
 #endif
 
    /* check for MESA_GAMMA environment variable */
-   gamma = getenv("MESA_GAMMA");
+   gamma = _mesa_getenv("MESA_GAMMA");
    if (gamma) {
       v->RedGamma = v->GreenGamma = v->BlueGamma = 0.0;
       sscanf( gamma, "%f %f %f", &v->RedGamma, &v->GreenGamma, &v->BlueGamma );
@@ -1620,11 +1612,10 @@ void XMesaDestroyVisual( XMesaVisual v )
  */
 XMesaContext XMesaCreateContext( XMesaVisual v, XMesaContext share_list )
 {
+   static GLboolean firstTime = GL_TRUE;
    XMesaContext c;
    GLcontext *ctx;
-   GLboolean direct = GL_TRUE; /* XXXX */
-   /* NOT_DONE: should this be GL_FALSE??? */
-   static GLboolean firstTime = GL_TRUE;
+   GLboolean direct = GL_TRUE; /* not really */
 
    if (firstTime) {
       _glthread_INIT_MUTEX(_xmesa_lock);
@@ -1638,7 +1629,7 @@ XMesaContext XMesaCreateContext( XMesaVisual v, XMesaContext share_list )
 
    ctx = c->gl_ctx = _mesa_create_context( &v->mesa_visual,
                       share_list ? share_list->gl_ctx : (GLcontext *) NULL,
-                      (void *) c, direct );
+                      (void *) c, direct);
    if (!c->gl_ctx) {
       FREE(c);
       return NULL;
@@ -1646,6 +1637,7 @@ XMesaContext XMesaCreateContext( XMesaVisual v, XMesaContext share_list )
 
    _mesa_enable_sw_extensions(ctx);
    _mesa_enable_1_3_extensions(ctx);
+   /*_mesa_enable_1_4_extensions(ctx);*/
 
    if (CHECK_BYTE_ORDER(v)) {
       c->swapbytes = GL_FALSE;
@@ -1655,6 +1647,8 @@ XMesaContext XMesaCreateContext( XMesaVisual v, XMesaContext share_list )
    }
 
    c->xm_visual = v;
+   c->xm_draw_buffer = NULL;   /* set later by XMesaMakeCurrent */
+   c->xm_read_buffer = NULL;   /* set later by XMesaMakeCurrent */
    c->xm_buffer = NULL;   /* set later by XMesaMakeCurrent */
    c->display = v->display;
    c->pixelformat = v->dithered_pf;      /* Dithering is enabled by default */
@@ -1674,12 +1668,6 @@ XMesaContext XMesaCreateContext( XMesaVisual v, XMesaContext share_list )
     */
    xmesa_init_pointers( ctx );
 
-
-   /* Run the config file
-    */
-   _mesa_read_config_file( ctx );
-
-
    return c;
 }
 
@@ -1689,8 +1677,8 @@ XMesaContext XMesaCreateContext( XMesaVisual v, XMesaContext share_list )
 void XMesaDestroyContext( XMesaContext c )
 {
 #ifdef FX
-   if (c->xm_buffer && c->xm_buffer->FXctx)
-      fxMesaDestroyContext(c->xm_buffer->FXctx);
+   if (c->xm_draw_buffer && c->xm_buffer->FXctx)
+      fxMesaDestroyContext(c->xm_draw_buffer->FXctx);
 #endif
    if (c->gl_ctx) {
       _swsetup_DestroyContext( c->gl_ctx );
@@ -1744,14 +1732,14 @@ XMesaBuffer XMesaCreateWindowBuffer2( XMesaVisual v, XMesaWindow w,
 
    if (GET_VISUAL_DEPTH(v) != attr.depth) {
 #endif
-      if (getenv("MESA_DEBUG")) {
+      if (_mesa_getenv("MESA_DEBUG")) {
          fprintf(stderr, "XMesaCreateWindowBuffer: depth mismatch between visual and window!\n");
       }
       return NULL;
    }
 
    b->xm_visual = v;
-   b->pixmap_flag = GL_FALSE;
+   b->type = WINDOW;
    b->display = v->display;
 #ifdef XFree86Server
    b->cmap = (ColormapPtr)LookupIDByType(wColormap(w), RT_COLORMAP);
@@ -1760,7 +1748,7 @@ XMesaBuffer XMesaCreateWindowBuffer2( XMesaVisual v, XMesaWindow w,
       b->cmap = attr.colormap;
    }
    else {
-      if (getenv("MESA_DEBUG")) {
+      if (_mesa_getenv("MESA_DEBUG")) {
          fprintf(stderr, "Window %u has no colormap!\n", (unsigned int) w);
       }
       /* this is weird, a window w/out a colormap!? */
@@ -1796,7 +1784,7 @@ XMesaBuffer XMesaCreateWindowBuffer2( XMesaVisual v, XMesaWindow w,
    }
 
 #ifdef FX
-   fxEnvVar = getenv("MESA_GLX_FX");
+   fxEnvVar = _mesa_getenv("MESA_GLX_FX");
    if (fxEnvVar) {
      if (fxEnvVar[0]!='d') {
        int attribs[100];
@@ -1897,7 +1885,7 @@ XMesaBuffer XMesaCreatePixmapBuffer( XMesaVisual v,
    assert(v);
 
    b->xm_visual = v;
-   b->pixmap_flag = GL_TRUE;
+   b->type = PIXMAP;
    b->display = v->display;
    b->cmap = cmap;
 
@@ -1934,21 +1922,29 @@ XMesaBuffer XMesaCreatePixmapBuffer( XMesaVisual v,
 
 
 
-#if 0 /* not done */
 XMesaBuffer XMesaCreatePBuffer( XMesaVisual v, XMesaColormap cmap,
                                 unsigned int width, unsigned int height )
 {
+#ifdef XFree86Server
+   return 0;
+#else
    int client = 0;
+   XMesaWindow root;
+   XMesaDrawable drawable;  /* X Pixmap Drawable */
    XMesaBuffer b = alloc_xmesa_buffer();
    if (!b) {
       return NULL;
    }
 
    b->xm_visual = v;
-   b->pbuffer_flag = GL_TRUE;
+   b->type = PBUFFER;
    b->display = v->display;
    b->cmap = cmap;
 
+   /* allocate pixmap for front buffer */
+   root = RootWindow( v->display, v->visinfo->screen );
+   drawable = XCreatePixmap( v->display, root, width, height, v->visinfo->depth );
+
    /* determine back buffer implementation */
    if (v->mesa_visual.doubleBufferMode) {
       if (v->ximage_flag) {
@@ -1972,14 +1968,14 @@ XMesaBuffer XMesaCreatePBuffer( XMesaVisual v, XMesaColormap cmap,
                                 v->mesa_visual.alphaBits > 0 );
 
    if (!initialize_visual_and_buffer(client, v, b, v->mesa_visual.rgbMode,
-				     0, cmap)) {
+				     drawable, cmap)) {
       free_xmesa_buffer(client, b);
       return NULL;
    }
 
    return b;
-}
 #endif
+}
 
 
 
@@ -2052,24 +2048,24 @@ GLboolean XMesaMakeCurrent2( XMesaContext c, XMesaBuffer drawBuffer,
       if (drawBuffer->FXctx) {
          fxMesaMakeCurrent(drawBuffer->FXctx);
 
-         c->xm_buffer = drawBuffer;
+         c->xm_draw_buffer = drawBuffer;
          c->xm_read_buffer = readBuffer;
-         c->use_read_buffer = (drawBuffer != readBuffer);
+         c->xm_buffer = drawBuffer;
 
          return GL_TRUE;
       }
 #endif
       if (c->gl_ctx == _mesa_get_current_context()
-          && c->xm_buffer == drawBuffer
+          && c->xm_draw_buffer == drawBuffer
           && c->xm_read_buffer == readBuffer
-          && c->xm_buffer->wasCurrent) {
+          && c->xm_draw_buffer->wasCurrent) {
          /* same context and buffer, do nothing */
          return GL_TRUE;
       }
 
-      c->xm_buffer = drawBuffer;
+      c->xm_draw_buffer = drawBuffer;
       c->xm_read_buffer = readBuffer;
-      c->use_read_buffer = (drawBuffer != readBuffer);
+      c->xm_buffer = drawBuffer;
 
       _mesa_make_current2(c->gl_ctx,
                           &drawBuffer->mesa_buffer,
@@ -2093,11 +2089,11 @@ GLboolean XMesaMakeCurrent2( XMesaContext c, XMesaBuffer drawBuffer,
                                                c->clearcolor[2],
                                                c->clearcolor[3],
                                                c->xm_visual->undithered_pf);
-         XMesaSetForeground(c->display, c->xm_buffer->cleargc, c->clearpixel);
+         XMesaSetForeground(c->display, c->xm_draw_buffer->cleargc, c->clearpixel);
       }
 
       /* Solution to Stephane Rehel's problem with glXReleaseBuffersMESA(): */
-      c->xm_buffer->wasCurrent = GL_TRUE;
+      c->xm_draw_buffer->wasCurrent = GL_TRUE;
    }
    else {
       /* Detach */
@@ -2135,7 +2131,7 @@ XMesaBuffer XMesaGetCurrentBuffer( void )
    GET_CURRENT_CONTEXT(ctx);
    if (ctx) {
       XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-      return xmesa->xm_buffer;
+      return xmesa->xm_draw_buffer;
    }
    else {
       return 0;
@@ -2149,7 +2145,7 @@ XMesaBuffer XMesaGetCurrentReadBuffer( void )
    GET_CURRENT_CONTEXT(ctx);
    if (ctx) {
       XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-      return xmesa->xm_buffer;
+      return xmesa->xm_read_buffer;
    }
    else {
       return 0;
@@ -2161,7 +2157,7 @@ GLboolean XMesaForceCurrent(XMesaContext c)
 {
    if (c) {
       if (c->gl_ctx != _mesa_get_current_context()) {
-	 _mesa_make_current(c->gl_ctx, &c->xm_buffer->mesa_buffer);
+	 _mesa_make_current(c->gl_ctx, &c->xm_draw_buffer->mesa_buffer);
       }
    }
    else {
@@ -2185,7 +2181,7 @@ GLboolean XMesaLoseCurrent(XMesaContext c)
 GLboolean XMesaSetFXmode( GLint mode )
 {
 #ifdef FX
-   const char *fx = getenv("MESA_GLX_FX");
+   const char *fx = _mesa_getenv("MESA_GLX_FX");
    if (fx && fx[0] != 'd') {
       GET_CURRENT_CONTEXT(ctx);
       GrHwConfiguration hw;
@@ -2200,15 +2196,15 @@ GLboolean XMesaSetFXmode( GLint mode )
       if (ctx) {
          XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
          if (mode == XMESA_FX_WINDOW) {
-	    if (xmesa->xm_buffer->FXisHackUsable) {
+	    if (xmesa->xm_draw_buffer->FXisHackUsable) {
 	       FX_grSstControl(GR_CONTROL_DEACTIVATE);
-	       xmesa->xm_buffer->FXwindowHack = GL_TRUE;
+	       xmesa->xm_draw_buffer->FXwindowHack = GL_TRUE;
 	       return GL_TRUE;
 	    }
 	 }
 	 else if (mode == XMESA_FX_FULLSCREEN) {
 	    FX_grSstControl(GR_CONTROL_ACTIVATE);
-	    xmesa->xm_buffer->FXwindowHack = GL_FALSE;
+	    xmesa->xm_draw_buffer->FXwindowHack = GL_FALSE;
 	    return GL_TRUE;
 	 }
 	 else {
@@ -2239,8 +2235,6 @@ static void FXgetImage( XMesaBuffer b )
    unsigned int bw, depth, width, height;
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
 
-   assert(xmesa->xm_buffer->FXctx);
-
 #ifdef XFree86Server
    x = b->frontbuffer->x;
    y = b->frontbuffer->y;
@@ -2248,37 +2242,37 @@ static void FXgetImage( XMesaBuffer b )
    height = b->frontbuffer->height;
    depth = b->frontbuffer->depth;
 #else
-   XGetGeometry( xmesa->xm_visual->display, b->frontbuffer,
+   XGetGeometry( b->xm_visual->display, b->frontbuffer,
                  &root, &xpos, &ypos, &width, &height, &bw, &depth);
 #endif
    if (b->width != width || b->height != height) {
-      b->width = MIN2((int)width, xmesa->xm_buffer->FXctx->width);
-      b->height = MIN2((int)height, xmesa->xm_buffer->FXctx->height);
+      b->width = MIN2((int)width, b->FXctx->width);
+      b->height = MIN2((int)height, b->FXctx->height);
       if (b->width & 1)
          b->width--;  /* prevent odd width */
       xmesa_alloc_back_buffer( b );
    }
 
    grLfbWriteColorFormat(GR_COLORFORMAT_ARGB);
-   if (xmesa->xm_visual->undithered_pf==PF_5R6G5B) {
+   if (b->xm_visual->undithered_pf==PF_5R6G5B) {
       /* Special case: 16bpp RGB */
       grLfbReadRegion( GR_BUFFER_FRONTBUFFER,       /* src buffer */
-                       0, xmesa->xm_buffer->FXctx->height - b->height,  /*pos*/
+                       0, b->FXctx->height - b->height,  /*pos*/
                        b->width, b->height,         /* size */
                        b->width * sizeof(GLushort), /* stride */
                        b->backimage->data);         /* dest buffer */
    }
-   else if (xmesa->xm_visual->dithered_pf==PF_DITHER
-	    && GET_VISUAL_DEPTH(xmesa->xm_visual)==8) {
+   else if (b->xm_visual->dithered_pf==PF_DITHER
+	    && GET_VISUAL_DEPTH(b->xm_visual)==8) {
       /* Special case: 8bpp RGB */
       for (y=0;y<b->height;y++) {
-         GLubyte *ptr = (GLubyte*) xmesa->xm_buffer->backimage->data
-                        + xmesa->xm_buffer->backimage->bytes_per_line * y;
+         GLubyte *ptr = (GLubyte*) b->backimage->data
+                        + b->backimage->bytes_per_line * y;
          XDITHER_SETUP(y);
 
          /* read row from 3Dfx frame buffer */
          grLfbReadRegion( GR_BUFFER_FRONTBUFFER,
-                          0, xmesa->xm_buffer->FXctx->height-(b->height-y),
+                          0, b->FXctx->height-(b->height-y),
                           b->width, 1,
                           0,
                           pixbuf );
@@ -2297,7 +2291,7 @@ static void FXgetImage( XMesaBuffer b )
       for (y=0;y<b->height;y++) {
          /* read row from 3Dfx frame buffer */
          grLfbReadRegion( GR_BUFFER_FRONTBUFFER,
-                          0, xmesa->xm_buffer->FXctx->height-(b->height-y),
+                          0, b->FXctx->height-(b->height-y),
                           b->width, 1,
                           0,
                           pixbuf );
@@ -2309,7 +2303,8 @@ static void FXgetImage( XMesaBuffer b )
 					       (pixbuf[x] & 0xf800) >> 8,
 					       (pixbuf[x] & 0x07e0) >> 3,
 					       (pixbuf[x] & 0x001f) << 3,
-					       0xff, xmesa->pixelformat));
+					       0xff,
+                                               b->xm_visual->undithered_pf));
          }
       }
    }
@@ -2330,7 +2325,7 @@ void XMesaSwapBuffers( XMesaBuffer b )
     * we have to flush any pending rendering commands first.
     */
    if (ctx && ctx->DrawBuffer == &(b->mesa_buffer))
-      _mesa_swapbuffers(ctx);
+      _mesa_notifySwapBuffers(ctx);
 
    if (b->db_state) {
 #ifdef FX
@@ -2396,7 +2391,7 @@ void XMesaCopySubBuffer( XMesaBuffer b, int x, int y, int width, int height )
     * we have to flush any pending rendering commands first.
     */
    if (ctx && ctx->DrawBuffer == &(b->mesa_buffer))
-      _mesa_swapbuffers(ctx);
+      _mesa_notifySwapBuffers(ctx);
 
    if (b->db_state) {
       int yTop = b->height - y - height;
@@ -2515,7 +2510,7 @@ const char *XMesaGetString( XMesaContext c, int name )
 {
    (void) c;
    if (name==XMESA_VERSION) {
-      return "4.0.3";
+      return "3.5";
    }
    else if (name==XMESA_EXTENSIONS) {
       return "";
@@ -2549,7 +2544,7 @@ void XMesaGarbageCollect( void )
    XMesaBuffer b, next;
    for (b=XMesaBufferList; b; b=next) {
       next = b->Next;
-      if (b->display && b->frontbuffer && !b->pixmap_flag) {
+      if (b->display && b->frontbuffer && b->type == WINDOW) {
 #ifdef XFree86Server
 	 /* NOT_NEEDED */
 #else
