@@ -1,4 +1,4 @@
-/* $Id: enums.c,v 1.20 2001/06/08 20:10:55 brianp Exp $ */
+/* $Id: enums.c,v 1.23 2002/10/25 21:06:26 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -28,13 +28,9 @@
  */
 
 
-#ifdef PC_HEADER
-#include "all.h"
-#else
 #include "glheader.h"
 #include "enums.h"
-#include "mem.h"
-#endif
+#include "imports.h"
 
 
 typedef struct {
@@ -857,7 +853,7 @@ static int sorted = 0;
 
 static int compar_name( const enum_elt *a, const enum_elt *b )
 {
-   return strcmp(a->c, b->c);
+   return _mesa_strcmp(a->c, b->c);
 }
 
 
@@ -928,7 +924,7 @@ const char *_mesa_lookup_enum_by_nr( int nr )
    }
    else {
       /* this isn't re-entrant safe, no big deal here */
-      sprintf(token_tmp, "0x%x", nr);
+      _mesa_sprintf(token_tmp, "0x%x", nr);
       return token_tmp;
    }
 }
