@@ -1,8 +1,8 @@
-/* $Id: s_drawpix.c,v 1.22.2.3 2002/04/19 01:10:48 brianp Exp $ */
+/* $Id: s_drawpix.c,v 1.38 2002/10/24 23:57:24 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.0.3
+ * Version:  4.1
  *
  * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
@@ -31,13 +31,12 @@
 #include "convolve.h"
 #include "image.h"
 #include "macros.h"
-#include "mem.h"
+#include "imports.h"
 #include "mmath.h"
 #include "pixel.h"
 
 #include "s_context.h"
 #include "s_drawpix.h"
-#include "s_fog.h"
 #include "s_pixeltex.h"
 #include "s_span.h"
 #include "s_stencil.h"
@@ -103,15 +102,21 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
 {
    SWcontext *swrast = SWRAST_CONTEXT(ctx);
    const struct gl_pixelstore_attrib *unpack = &ctx->Unpack;
-   GLchan rgb[MAX_WIDTH][3];
-   GLchan rgba[MAX_WIDTH][4];
+   struct sw_span span;
+
+   INIT_SPAN(span, GL_BITMAP, 0, 0, SPAN_RGBA);
 
    if (!ctx->Current.RasterPosValid) {
       return GL_TRUE;      /* no-op */
    }
 
-   if ((SWRAST_CONTEXT(ctx)->_RasterMask&(~(SCISSOR_BIT|WINCLIP_BIT)))==0
-       && ctx->Texture._ReallyEnabled == 0
+   if (ctx->Depth.Test)
+      _mesa_span_default_z(ctx, &span);
+   if (ctx->Fog.Enabled)
+      _mesa_span_default_fog(ctx, &span);
+
+   if ((SWRAST_CONTEXT(ctx)->_RasterMask & ~CLIP_BIT) == 0
+       && ctx->Texture._EnabledUnits == 0
        && unpack->Alignment == 1
        && !unpack->SwapBytes
        && !unpack->LsbFirst) {
@@ -232,8 +237,11 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                /* with zooming */
                GLint row;
                for (row=0; row<drawHeight; row++) {
-                  _mesa_write_zoomed_rgba_span(ctx, drawWidth, destX, destY,
-                                  zSpan, 0, (CONST GLchan (*)[4]) src, zoomY0);
+                  span.x = destX;
+                  span.y = destY;
+                  span.end = drawWidth;
+                  _mesa_write_zoomed_rgba_span(ctx, &span,
+                                            (CONST GLchan (*)[4]) src, zoomY0);
                   src += rowLength * 4;
                   destY++;
                }
@@ -269,8 +277,11 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                /* with zooming */
                GLint row;
                for (row=0; row<drawHeight; row++) {
-                  _mesa_write_zoomed_rgb_span(ctx, drawWidth, destX, destY,
-                                  zSpan, 0, (CONST GLchan (*)[3]) src, zoomY0);
+                  span.x = destX;
+                  span.y = destY;
+                  span.end = drawWidth;
+                  _mesa_write_zoomed_rgb_span(ctx, &span, 
+                                            (CONST GLchan (*)[3]) src, zoomY0);
                   src += rowLength * 3;
                   destY++;
                }
@@ -290,12 +301,12 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                for (row=0; row<drawHeight; row++) {
                   GLint i;
 		  for (i=0;i<drawWidth;i++) {
-                     rgb[i][0] = src[i];
-                     rgb[i][1] = src[i];
-                     rgb[i][2] = src[i];
+                     span.array->rgb[i][0] = src[i];
+                     span.array->rgb[i][1] = src[i];
+                     span.array->rgb[i][2] = src[i];
 		  }
                   (*swrast->Driver.WriteRGBSpan)(ctx, drawWidth, destX, destY,
-                                              (CONST GLchan (*)[3]) rgb, NULL);
+                                   (CONST GLchan (*)[3]) span.array->rgb, NULL);
                   src += rowLength;
                   destY++;
                }
@@ -307,13 +318,13 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                for (row=0; row<drawHeight; row++) {
                   GLint i;
                   for (i=0;i<drawWidth;i++) {
-                     rgb[i][0] = src[i];
-                     rgb[i][1] = src[i];
-                     rgb[i][2] = src[i];
+                     span.array->rgb[i][0] = src[i];
+                     span.array->rgb[i][1] = src[i];
+                     span.array->rgb[i][2] = src[i];
                   }
                   destY--;
                   (*swrast->Driver.WriteRGBSpan)(ctx, drawWidth, destX, destY,
-                                              (CONST GLchan (*)[3]) rgb, NULL);
+                                              (CONST GLchan (*)[3]) span.array->rgb, NULL);
                   src += rowLength;
                }
             }
@@ -324,12 +335,15 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                for (row=0; row<drawHeight; row++) {
                   GLint i;
 		  for (i=0;i<drawWidth;i++) {
-                     rgb[i][0] = src[i];
-                     rgb[i][1] = src[i];
-                     rgb[i][2] = src[i];
+                     span.array->rgb[i][0] = src[i];
+                     span.array->rgb[i][1] = src[i];
+                     span.array->rgb[i][2] = src[i];
 		  }
-                  _mesa_write_zoomed_rgb_span(ctx, drawWidth, destX, destY,
-                                  zSpan, 0, (CONST GLchan (*)[3]) rgb, zoomY0);
+                  span.x = destX;
+                  span.y = destY;
+                  span.end = drawWidth;
+                  _mesa_write_zoomed_rgb_span(ctx, &span,
+                                 (CONST GLchan (*)[3]) span.array->rgb, zoomY0);
                   src += rowLength;
                   destY++;
                }
@@ -350,13 +364,13 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                   GLint i;
                   GLchan *ptr = src;
 		  for (i=0;i<drawWidth;i++) {
-                     rgba[i][0] = *ptr;
-                     rgba[i][1] = *ptr;
-                     rgba[i][2] = *ptr++;
-                     rgba[i][3] = *ptr++;
+                     span.array->rgba[i][0] = *ptr;
+                     span.array->rgba[i][1] = *ptr;
+                     span.array->rgba[i][2] = *ptr++;
+                     span.array->rgba[i][3] = *ptr++;
 		  }
                   (*swrast->Driver.WriteRGBASpan)(ctx, drawWidth, destX, destY,
-                                             (CONST GLchan (*)[4]) rgba, NULL);
+                                             (CONST GLchan (*)[4]) span.array->rgba, NULL);
                   src += rowLength*2;
                   destY++;
                }
@@ -369,14 +383,14 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                   GLint i;
                   GLchan *ptr = src;
                   for (i=0;i<drawWidth;i++) {
-                     rgba[i][0] = *ptr;
-                     rgba[i][1] = *ptr;
-                     rgba[i][2] = *ptr++;
-                     rgba[i][3] = *ptr++;
+                     span.array->rgba[i][0] = *ptr;
+                     span.array->rgba[i][1] = *ptr;
+                     span.array->rgba[i][2] = *ptr++;
+                     span.array->rgba[i][3] = *ptr++;
                   }
                   destY--;
                   (*swrast->Driver.WriteRGBASpan)(ctx, drawWidth, destX, destY,
-                                             (CONST GLchan (*)[4]) rgba, NULL);
+                                             (CONST GLchan (*)[4]) span.array->rgba, NULL);
                   src += rowLength*2;
                }
             }
@@ -388,13 +402,16 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                   GLchan *ptr = src;
                   GLint i;
 		  for (i=0;i<drawWidth;i++) {
-                     rgba[i][0] = *ptr;
-                     rgba[i][1] = *ptr;
-                     rgba[i][2] = *ptr++;
-                     rgba[i][3] = *ptr++;
+                     span.array->rgba[i][0] = *ptr;
+                     span.array->rgba[i][1] = *ptr;
+                     span.array->rgba[i][2] = *ptr++;
+                     span.array->rgba[i][3] = *ptr++;
 		  }
-                  _mesa_write_zoomed_rgba_span(ctx, drawWidth, destX, destY,
-                                 zSpan, 0, (CONST GLchan (*)[4]) rgba, zoomY0);
+                  span.x = destX;
+                  span.y = destY;
+                  span.end = drawWidth;
+                  _mesa_write_zoomed_rgba_span(ctx, &span,
+                               (CONST GLchan (*)[4]) span.array->rgba, zoomY0);
                   src += rowLength*2;
                   destY++;
                }
@@ -411,10 +428,9 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                GLint row;
                for (row=0; row<drawHeight; row++) {
                   ASSERT(drawWidth < MAX_WIDTH);
-                  _mesa_map_ci8_to_rgba(ctx, drawWidth, src, rgba);
+                  _mesa_map_ci8_to_rgba(ctx, drawWidth, src, span.array->rgba);
                   (*swrast->Driver.WriteRGBASpan)(ctx, drawWidth, destX, destY,
-                                               (const GLchan (*)[4]) rgba,
-					       NULL);
+                                  (const GLchan (*)[4]) span.array->rgba, NULL);
                   src += rowLength;
                   destY++;
                }
@@ -425,11 +441,10 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                GLint row;
                for (row=0; row<drawHeight; row++) {
                   ASSERT(drawWidth < MAX_WIDTH);
-                  _mesa_map_ci8_to_rgba(ctx, drawWidth, src, rgba);
+                  _mesa_map_ci8_to_rgba(ctx, drawWidth, src, span.array->rgba);
                   destY--;
                   (*swrast->Driver.WriteRGBASpan)(ctx, drawWidth, destX, destY,
-                                               (CONST GLchan (*)[4]) rgba,
-                                               NULL);
+                                 (CONST GLchan (*)[4]) span.array->rgba, NULL);
                   src += rowLength;
                }
                return GL_TRUE;
@@ -439,9 +454,12 @@ fast_draw_pixels(GLcontext *ctx, GLint x, GLint y,
                GLint row;
                for (row=0; row<drawHeight; row++) {
                   ASSERT(drawWidth < MAX_WIDTH);
-                  _mesa_map_ci8_to_rgba(ctx, drawWidth, src, rgba);
-                  _mesa_write_zoomed_rgba_span(ctx, drawWidth, destX, destY,
-                                 zSpan, 0, (CONST GLchan (*)[4]) rgba, zoomY0);
+                  _mesa_map_ci8_to_rgba(ctx, drawWidth, src, span.array->rgba);
+                  span.x = destX;
+                  span.y = destY;
+                  span.end = drawWidth;
+                  _mesa_write_zoomed_rgba_span(ctx, &span,
+                               (CONST GLchan (*)[4]) span.array->rgba, zoomY0);
                   src += rowLength;
                   destY++;
                }
@@ -489,47 +507,33 @@ draw_index_pixels( GLcontext *ctx, GLint x, GLint y,
 {
    const GLboolean zoom = ctx->Pixel.ZoomX!=1.0 || ctx->Pixel.ZoomY!=1.0;
    const GLint desty = y;
-   GLint row, drawWidth;
-   GLdepth zspan[MAX_WIDTH];
-   GLfloat fogSpan[MAX_WIDTH];
+   GLint row, drawWidth = (width > MAX_WIDTH) ? MAX_WIDTH : width;
+   struct sw_span span;
 
-   drawWidth = (width > MAX_WIDTH) ? MAX_WIDTH : width;
+   INIT_SPAN(span, GL_BITMAP, drawWidth, 0, SPAN_INDEX);
 
-   /* Fragment depth values */
-   if (ctx->Depth.Test || ctx->Fog.Enabled) {
-      GLdepth zval = (GLdepth) (ctx->Current.RasterPos[2] * ctx->DepthMaxF);
-      GLfloat fog;
-      GLint i;
-
-      if (ctx->Fog.FogCoordinateSource == GL_FOG_COORDINATE_EXT)
-         fog = _mesa_z_to_fogfactor(ctx, ctx->Current.RasterFogCoord);
-      else
-         fog = _mesa_z_to_fogfactor(ctx, ctx->Current.RasterDistance);
-
-      for (i = 0; i < drawWidth; i++) {
-	 zspan[i] = zval;
-         fogSpan[i] = fog;
-      }
-   }
+   if (ctx->Depth.Test)
+      _mesa_span_default_z(ctx, &span);
+   if (ctx->Fog.Enabled)
+      _mesa_span_default_fog(ctx, &span);
 
    /*
     * General solution
     */
    for (row = 0; row < height; row++, y++) {
-      GLuint indexes[MAX_WIDTH];
       const GLvoid *source = _mesa_image_address(&ctx->Unpack,
                     pixels, width, height, GL_COLOR_INDEX, type, 0, row, 0);
-      _mesa_unpack_index_span(ctx, drawWidth, GL_UNSIGNED_INT, indexes,
+      _mesa_unpack_index_span(ctx, drawWidth, GL_UNSIGNED_INT,
+                              span.array->index,
                               type, source, &ctx->Unpack,
                               ctx->_ImageTransferState);
-      if (zoom) {
-         _mesa_write_zoomed_index_span(ctx, drawWidth, x, y, zspan, fogSpan,
-                                       indexes, desty);
-      }
-      else {
-         _mesa_write_index_span(ctx, drawWidth, x, y, zspan, fogSpan, indexes,
-                                NULL, GL_BITMAP);
-      }
+      span.x = x;
+      span.y = y;
+      span.end = drawWidth;
+      if (zoom)
+         _mesa_write_zoomed_index_span(ctx, &span, desty);
+      else
+         _mesa_write_index_span(ctx, &span);
    }
 }
 
@@ -594,7 +598,6 @@ draw_stencil_pixels( GLcontext *ctx, GLint x, GLint y,
 }
 
 
-
 /*
  * Do a glDrawPixels of depth values.
  */
@@ -606,9 +609,10 @@ draw_depth_pixels( GLcontext *ctx, GLint x, GLint y,
    const GLboolean bias_or_scale = ctx->Pixel.DepthBias!=0.0 || ctx->Pixel.DepthScale!=1.0;
    const GLboolean zoom = ctx->Pixel.ZoomX!=1.0 || ctx->Pixel.ZoomY!=1.0;
    const GLint desty = y;
-   GLchan rgba[MAX_WIDTH][4];
-   GLuint ispan[MAX_WIDTH];
    GLint drawWidth = (width > MAX_WIDTH) ? MAX_WIDTH : width;
+   struct sw_span span;
+
+   INIT_SPAN(span, GL_BITMAP, drawWidth, 0, SPAN_Z);
 
    if (type != GL_BYTE
        && type != GL_UNSIGNED_BYTE
@@ -621,62 +625,53 @@ draw_depth_pixels( GLcontext *ctx, GLint x, GLint y,
       return;
    }
 
-   /* Colors or indexes */
-   if (ctx->Visual.rgbMode) {
-      GLint i;
-      GLint r, g, b, a;
-      UNCLAMPED_FLOAT_TO_CHAN(r, ctx->Current.RasterColor[0]);
-      UNCLAMPED_FLOAT_TO_CHAN(g, ctx->Current.RasterColor[1]);
-      UNCLAMPED_FLOAT_TO_CHAN(b, ctx->Current.RasterColor[2]);
-      UNCLAMPED_FLOAT_TO_CHAN(a, ctx->Current.RasterColor[3]);
-      for (i = 0; i < drawWidth; i++) {
-         rgba[i][RCOMP] = r;
-         rgba[i][GCOMP] = g;
-         rgba[i][BCOMP] = b;
-         rgba[i][ACOMP] = a;
-      }
-   }
-   else {
-      GLint i;
-      for (i = 0; i < drawWidth; i++) {
-	 ispan[i] = ctx->Current.RasterIndex;
-      }
-   }
+   _mesa_span_default_color(ctx, &span);
+
+   if (ctx->Fog.Enabled)
+      _mesa_span_default_fog(ctx, &span);
+   if (ctx->Texture._EnabledUnits)
+      _mesa_span_default_texcoords(ctx, &span);
 
    if (type==GL_UNSIGNED_SHORT && ctx->Visual.depthBits == 16
        && !bias_or_scale && !zoom && ctx->Visual.rgbMode) {
       /* Special case: directly write 16-bit depth values */
       GLint row;
-      for (row = 0; row < height; row++, y++) {
-         GLdepth zspan[MAX_WIDTH];
+      span.x = x;
+      span.y = y;
+      span.end = drawWidth;
+      for (row = 0; row < height; row++, span.y++) {
          const GLushort *zptr = (const GLushort *)
             _mesa_image_address(&ctx->Unpack, pixels, width, height,
                                 GL_DEPTH_COMPONENT, type, 0, row, 0);
          GLint i;
-         for (i = 0; i < width; i++)
-            zspan[i] = zptr[i];
-         _mesa_write_rgba_span(ctx, width, x, y, zspan, 0, rgba,
-                               NULL, GL_BITMAP);
+         for (i = 0; i < drawWidth; i++)
+            span.array->z[i] = zptr[i];
+         _mesa_write_rgba_span(ctx, &span);
       }
    }
    else if (type==GL_UNSIGNED_INT && ctx->Visual.depthBits == 32
        && !bias_or_scale && !zoom && ctx->Visual.rgbMode) {
       /* Special case: directly write 32-bit depth values */
       GLint row;
-      for (row = 0; row < height; row++, y++) {
+      span.x = x;
+      span.y = y;
+      span.end = drawWidth;
+      for (row = 0; row < height; row++, span.y++) {
          const GLuint *zptr = (const GLuint *)
             _mesa_image_address(&ctx->Unpack, pixels, width, height,
                                 GL_DEPTH_COMPONENT, type, 0, row, 0);
-         _mesa_write_rgba_span(ctx, width, x, y, zptr, 0, rgba,
-                               NULL, GL_BITMAP);
+         MEMCPY(span.array->z, zptr, drawWidth * sizeof(GLdepth));
+         _mesa_write_rgba_span(ctx, &span);
       }
    }
    else {
       /* General case */
       GLint row;
-      for (row = 0; row < height; row++, y++) {
+      span.x = x;
+      span.y = y;
+      span.end = drawWidth;
+      for (row = 0; row < height; row++, span.y++) {
          GLfloat fspan[MAX_WIDTH];
-         GLdepth zspan[MAX_WIDTH];
          const GLvoid *src = _mesa_image_address(&ctx->Unpack,
                 pixels, width, height, GL_DEPTH_COMPONENT, type, 0, row, 0);
          _mesa_unpack_depth_span( ctx, drawWidth, fspan, type, src,
@@ -686,31 +681,25 @@ draw_depth_pixels( GLcontext *ctx, GLint x, GLint y,
             const GLfloat zs = ctx->DepthMaxF;
             GLint i;
             for (i = 0; i < drawWidth; i++) {
-               zspan[i] = (GLdepth) (fspan[i] * zs + 0.5F);
+               span.array->z[i] = (GLdepth) (fspan[i] * zs + 0.5F);
             }
          }
-
          if (ctx->Visual.rgbMode) {
             if (zoom) {
-               _mesa_write_zoomed_rgba_span(ctx, width, x, y, zspan, 0,
-                                            (const GLchan (*)[4]) rgba, desty);
-            }
-            else {
-               _mesa_write_rgba_span(ctx, width, x, y, zspan, 0,
-                                     rgba, NULL, GL_BITMAP);
+               abort();
+               _mesa_write_zoomed_rgba_span(ctx, &span,
+                                 (const GLchan (*)[4]) span.array->rgba, desty);
             }
+            else
+               _mesa_write_rgba_span(ctx, &span);
          }
          else {
-            if (zoom) {
-               _mesa_write_zoomed_index_span(ctx, width, x, y, zspan, 0,
-                                             ispan, GL_BITMAP);
-            }
-            else {
-               _mesa_write_index_span(ctx, width, x, y, zspan, 0,
-                                      ispan, NULL, GL_BITMAP);
-            }
+               abort();
+            if (zoom)
+               _mesa_write_zoomed_index_span(ctx, &span, desty);
+            else
+               _mesa_write_index_span(ctx, &span);
          }
-
       }
    }
 }
@@ -728,11 +717,12 @@ draw_rgba_pixels( GLcontext *ctx, GLint x, GLint y,
    const struct gl_pixelstore_attrib *unpack = &ctx->Unpack;
    const GLboolean zoom = ctx->Pixel.ZoomX!=1.0 || ctx->Pixel.ZoomY!=1.0;
    const GLint desty = y;
-   GLdepth zspan[MAX_WIDTH];
-   GLfloat fogSpan[MAX_WIDTH];
    GLboolean quickDraw;
    GLfloat *convImage = NULL;
    GLuint transferOps = ctx->_ImageTransferState;
+   struct sw_span span;
+
+   INIT_SPAN(span, GL_BITMAP, 0, 0, SPAN_RGBA);
 
    if (!_mesa_is_legal_format_and_type(format, type)) {
       _mesa_error(ctx, GL_INVALID_ENUM, "glDrawPixels(format or type)");
@@ -743,24 +733,12 @@ draw_rgba_pixels( GLcontext *ctx, GLint x, GLint y,
    if (fast_draw_pixels(ctx, x, y, width, height, format, type, pixels))
       return;
 
-   /* Fragment depth values */
-   if (ctx->Depth.Test || ctx->Fog.Enabled) {
-      /* fill in array of z values */
-      GLdepth z = (GLdepth) (ctx->Current.RasterPos[2] * ctx->DepthMaxF);
-      GLfloat fog;
-      GLint i;
-
-      if (ctx->Fog.FogCoordinateSource == GL_FOG_COORDINATE_EXT)
-         fog = _mesa_z_to_fogfactor(ctx, ctx->Current.RasterFogCoord);
-      else
-         fog = _mesa_z_to_fogfactor(ctx, ctx->Current.RasterDistance);
-
-      for (i=0;i<width;i++) {
-	 zspan[i] = z;
-         fogSpan[i] = fog;
-      }
-   }
-
+   if (ctx->Depth.Test)
+      _mesa_span_default_z(ctx, &span);
+   if (ctx->Fog.Enabled)
+      _mesa_span_default_fog(ctx, &span);
+   if (ctx->Texture._EnabledUnits)
+      _mesa_span_default_texcoords(ctx, &span);
 
    if (SWRAST_CONTEXT(ctx)->_RasterMask == 0 && !zoom && x >= 0 && y >= 0
        && x + width <= (GLint) ctx->DrawBuffer->Width
@@ -827,61 +805,44 @@ draw_rgba_pixels( GLcontext *ctx, GLint x, GLint y,
     * General solution
     */
    {
-      GLchan rgba[MAX_WIDTH][4];
       GLint row;
       if (width > MAX_WIDTH)
          width = MAX_WIDTH;
+
       for (row = 0; row < height; row++, y++) {
          const GLvoid *source = _mesa_image_address(unpack,
                   pixels, width, height, format, type, 0, row, 0);
-         _mesa_unpack_chan_color_span(ctx, width, GL_RGBA, (GLchan *) rgba,
+
+         _mesa_unpack_chan_color_span(ctx, width, GL_RGBA,
+                                      (GLchan *) span.array->rgba,
                                       format, type, source, unpack,
                                       transferOps);
+
          if ((ctx->Pixel.MinMaxEnabled && ctx->MinMax.Sink) ||
              (ctx->Pixel.HistogramEnabled && ctx->Histogram.Sink))
             continue;
 
-         if (ctx->Texture._ReallyEnabled && ctx->Pixel.PixelTextureEnabled) {
-            GLchan primary_rgba[MAX_WIDTH][4];
-            GLuint unit;
-            DEFARRAY(GLfloat, s, MAX_WIDTH);  /* mac 32k limitation */
-            DEFARRAY(GLfloat, t, MAX_WIDTH);
-            DEFARRAY(GLfloat, r, MAX_WIDTH);
-            DEFARRAY(GLfloat, q, MAX_WIDTH);
-            CHECKARRAY(s, return);  /* mac 32k limitation */
-            CHECKARRAY(t, return);
-            CHECKARRAY(r, return);
-            CHECKARRAY(q, return);
-
-            /* XXX not sure how multitexture is supposed to work here */
-            MEMCPY(primary_rgba, rgba, 4 * width * sizeof(GLchan));
-
-            for (unit = 0; unit < ctx->Const.MaxTextureUnits; unit++) {
-               if (ctx->Texture.Unit[unit]._ReallyEnabled) {
-                  _mesa_pixeltexgen(ctx, width, (const GLchan (*)[4]) rgba,
-                                    s, t, r, q);
-                  _swrast_texture_fragments(ctx, unit, width, s, t, r, NULL,
-                                            (CONST GLchan (*)[4]) primary_rgba,
-                                            rgba);
-               }
-            }
-            UNDEFARRAY(s);  /* mac 32k limitation */
-            UNDEFARRAY(t);
-            UNDEFARRAY(r);
-            UNDEFARRAY(q);
+         if (ctx->Pixel.PixelTextureEnabled && ctx->Texture._EnabledUnits) {
+            span.end = width;
+            _swrast_pixel_texture(ctx, &span);
          }
 
          if (quickDraw) {
             (*swrast->Driver.WriteRGBASpan)(ctx, width, x, y,
-                                            (CONST GLchan (*)[4]) rgba, NULL);
+                                 (CONST GLchan (*)[4]) span.array->rgba, NULL);
          }
          else if (zoom) {
-            _mesa_write_zoomed_rgba_span(ctx, width, x, y, zspan, fogSpan,
-                                         (CONST GLchan (*)[4]) rgba, desty);
+            span.x = x;
+            span.y = y;
+            span.end = width;
+            _mesa_write_zoomed_rgba_span(ctx, &span,
+                                (CONST GLchan (*)[4]) span.array->rgba, desty);
          }
          else {
-            _mesa_write_rgba_span(ctx, (GLuint) width, x, y, zspan, fogSpan,
-                                  rgba, NULL, GL_BITMAP);
+            span.x = x;
+            span.y = y;
+            span.end = width;
+            _mesa_write_rgba_span(ctx, &span);
          }
       }
    }
@@ -907,7 +868,6 @@ _swrast_DrawPixels( GLcontext *ctx,
    SWcontext *swrast = SWRAST_CONTEXT(ctx);
    (void) unpack;
 
-
    if (swrast->NewState)
       _swrast_validate_derived( ctx );
 
@@ -945,3 +905,55 @@ _swrast_DrawPixels( GLcontext *ctx,
 
    RENDER_FINISH(swrast,ctx);
 }
+
+
+
+#if 0  /* experimental */
+/*
+ * Execute glDrawDepthPixelsMESA().
+ */
+void
+_swrast_DrawDepthPixelsMESA( GLcontext *ctx,
+                             GLint x, GLint y,
+                             GLsizei width, GLsizei height,
+                             GLenum colorFormat, GLenum colorType,
+                             const GLvoid *colors,
+                             GLenum depthType, const GLvoid *depths,
+                             const struct gl_pixelstore_attrib *unpack )
+{
+   SWcontext *swrast = SWRAST_CONTEXT(ctx);
+   (void) unpack;
+
+   if (swrast->NewState)
+      _swrast_validate_derived( ctx );
+
+   RENDER_START(swrast,ctx);
+
+   switch (colorFormat) {
+   case GL_COLOR_INDEX:
+      if (ctx->Visual.rgbMode)
+	 draw_rgba_pixels(ctx, x,y, width, height, colorFormat, colorType, colors);
+      else
+	 draw_index_pixels(ctx, x, y, width, height, colorType, colors);
+      break;
+   case GL_RED:
+   case GL_GREEN:
+   case GL_BLUE:
+   case GL_ALPHA:
+   case GL_LUMINANCE:
+   case GL_LUMINANCE_ALPHA:
+   case GL_RGB:
+   case GL_BGR:
+   case GL_RGBA:
+   case GL_BGRA:
+   case GL_ABGR_EXT:
+      draw_rgba_pixels(ctx, x, y, width, height, colorFormat, colorType, colors);
+      break;
+   default:
+      _mesa_error( ctx, GL_INVALID_ENUM,
+                   "glDrawDepthPixelsMESA(colorFormat)" );
+   }
+
+   RENDER_FINISH(swrast,ctx);
+}
+#endif
