@@ -1,10 +1,10 @@
-/* $Id: ss_context.c,v 1.15 2001/07/17 19:39:32 keithw Exp $ */
+/* $Id: ss_context.c,v 1.19 2002/10/24 23:57:25 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.1
  *
- * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,7 +28,7 @@
  */
 
 #include "glheader.h"
-#include "mem.h"
+#include "imports.h"
 #include "ss_context.h"
 #include "ss_triangle.h"
 #include "ss_vb.h"
@@ -49,7 +49,6 @@
 #define _SWSETUP_NEW_RENDERINDEX (_NEW_POLYGON|_NEW_LIGHT)
 
 
-
 GLboolean
 _swsetup_CreateContext( GLcontext *ctx )
 {
@@ -59,7 +58,7 @@ _swsetup_CreateContext( GLcontext *ctx )
    if (!swsetup)
       return GL_FALSE;
 
-   swsetup->verts = (SWvertex *) ALIGN_MALLOC( sizeof(SWvertex) * tnl->vb.Size,
+   swsetup->verts = (SWvertex *) ALIGN_CALLOC( sizeof(SWvertex) * tnl->vb.Size,
 					       32);
    if (!swsetup->verts) {
       FREE(swsetup);
@@ -99,8 +98,13 @@ static void
 _swsetup_RenderPrimitive( GLcontext *ctx, GLenum mode )
 {
    SWSETUP_CONTEXT(ctx)->render_prim = mode;
+   _swrast_render_primitive( ctx, mode );
 }
 
+/*
+ * We patch this function into tnl->Driver.Render.Start.
+ * It's called when we start rendering a vertex buffer.
+ */
 static void
 _swsetup_RenderStart( GLcontext *ctx )
 {
@@ -117,19 +121,17 @@ _swsetup_RenderStart( GLcontext *ctx )
 
    swsetup->NewState = 0;
 
-   if (swsetup->Driver.Start) 
-      swsetup->Driver.Start( ctx );
+   _swrast_render_start( ctx );
 }
 
+/*
+ * We patch this function into tnl->Driver.Render.Finish.
+ * It's called when we finish rendering a vertex buffer.
+ */
 static void
 _swsetup_RenderFinish( GLcontext *ctx )
 {
-   SScontext *swsetup = SWSETUP_CONTEXT(ctx);
-
-   _swrast_flush( ctx );
-
-   if (swsetup->Driver.Finish) 
-      swsetup->Driver.Finish( ctx );
+   _swrast_render_finish( ctx );
 }
 
 void
