@@ -1,10 +1,10 @@
-/* $Id: ss_context.h,v 1.9 2001/07/17 19:39:32 keithw Exp $ */
+/* $Id: ss_context.h,v 1.11 2002/10/04 17:37:47 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.1
  *
- * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -44,12 +44,6 @@ typedef struct {
     */
    struct gl_client_array ChanColor;
    struct gl_client_array ChanSecondaryColor;
-
-
-   struct {
-      void (*Start)( GLcontext * );
-      void (*Finish)( GLcontext * );
-   } Driver;
 } SScontext;
 
 #define SWSETUP_CONTEXT(ctx) ((SScontext *)ctx->swsetup_context)
