@@ -1,4 +1,4 @@
-/* $Id: texformat.c,v 1.11 2001/06/15 14:18:46 brianp Exp $ */
+/* $Id: texformat.c,v 1.17 2002/10/24 23:57:21 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -27,21 +27,16 @@
  *    Gareth Hughes <gareth@valinux.com>
  */
 
-#ifdef PC_HEADER
-#include "all.h"
-#else
 #include "glheader.h"
 #include "colormac.h"
 #include "context.h"
 #include "image.h"
-#include "mem.h"
+#include "imports.h"
 #include "mmath.h"
 #include "mtypes.h"
 #include "texformat.h"
 #include "teximage.h"
 #include "texstate.h"
-#include "swrast/s_span.h"
-#endif
 
 
 /* Texel fetch routines for all supported formats:
@@ -75,7 +70,6 @@ static void fetch_null_texel( const struct gl_texture_image *texImage,
 const struct gl_texture_format _mesa_texformat_rgba = {
    MESA_FORMAT_RGBA,			/* MesaFormat */
    GL_RGBA,				/* BaseFormat */
-   CHAN_TYPE,				/* Type */
    CHAN_BITS,				/* RedBits */
    CHAN_BITS,				/* GreenBits */
    CHAN_BITS,				/* BlueBits */
@@ -93,7 +87,6 @@ const struct gl_texture_format _mesa_texformat_rgba = {
 const struct gl_texture_format _mesa_texformat_rgb = {
    MESA_FORMAT_RGB,			/* MesaFormat */
    GL_RGB,				/* BaseFormat */
-   CHAN_TYPE,				/* Type */
    CHAN_BITS,				/* RedBits */
    CHAN_BITS,				/* GreenBits */
    CHAN_BITS,				/* BlueBits */
@@ -111,7 +104,6 @@ const struct gl_texture_format _mesa_texformat_rgb = {
 const struct gl_texture_format _mesa_texformat_alpha = {
    MESA_FORMAT_ALPHA,			/* MesaFormat */
    GL_ALPHA,				/* BaseFormat */
-   CHAN_TYPE,				/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -129,7 +121,6 @@ const struct gl_texture_format _mesa_texformat_alpha = {
 const struct gl_texture_format _mesa_texformat_luminance = {
    MESA_FORMAT_LUMINANCE,		/* MesaFormat */
    GL_LUMINANCE,			/* BaseFormat */
-   CHAN_TYPE,				/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -147,7 +138,6 @@ const struct gl_texture_format _mesa_texformat_luminance = {
 const struct gl_texture_format _mesa_texformat_luminance_alpha = {
    MESA_FORMAT_LUMINANCE_ALPHA,		/* MesaFormat */
    GL_LUMINANCE_ALPHA,			/* BaseFormat */
-   CHAN_TYPE,				/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -165,7 +155,6 @@ const struct gl_texture_format _mesa_texformat_luminance_alpha = {
 const struct gl_texture_format _mesa_texformat_intensity = {
    MESA_FORMAT_INTENSITY,		/* MesaFormat */
    GL_INTENSITY,			/* BaseFormat */
-   CHAN_TYPE,				/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -183,7 +172,6 @@ const struct gl_texture_format _mesa_texformat_intensity = {
 const struct gl_texture_format _mesa_texformat_color_index = {
    MESA_FORMAT_COLOR_INDEX,		/* MesaFormat */
    GL_COLOR_INDEX,			/* BaseFormat */
-   CHAN_TYPE,				/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -201,7 +189,6 @@ const struct gl_texture_format _mesa_texformat_color_index = {
 const struct gl_texture_format _mesa_texformat_depth_component = {
    MESA_FORMAT_DEPTH_COMPONENT,		/* MesaFormat */
    GL_DEPTH_COMPONENT,			/* BaseFormat */
-   GL_FLOAT,				/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -224,7 +211,6 @@ const struct gl_texture_format _mesa_texformat_depth_component = {
 const struct gl_texture_format _mesa_texformat_rgba8888 = {
    MESA_FORMAT_RGBA8888,		/* MesaFormat */
    GL_RGBA,				/* BaseFormat */
-   GL_UNSIGNED_INT_8_8_8_8,		/* Type */
    8,					/* RedBits */
    8,					/* GreenBits */
    8,					/* BlueBits */
@@ -242,7 +228,6 @@ const struct gl_texture_format _mesa_texformat_rgba8888 = {
 const struct gl_texture_format _mesa_texformat_argb8888 = {
    MESA_FORMAT_ARGB8888,		/* MesaFormat */
    GL_RGBA,				/* BaseFormat */
-   GL_UNSIGNED_INT_8_8_8_8_REV,		/* Type */
    8,					/* RedBits */
    8,					/* GreenBits */
    8,					/* BlueBits */
@@ -260,7 +245,6 @@ const struct gl_texture_format _mesa_texformat_argb8888 = {
 const struct gl_texture_format _mesa_texformat_rgb888 = {
    MESA_FORMAT_RGB888,			/* MesaFormat */
    GL_RGB,				/* BaseFormat */
-   GL_UNSIGNED_BYTE,			/* Type */
    8,					/* RedBits */
    8,					/* GreenBits */
    8,					/* BlueBits */
@@ -278,7 +262,6 @@ const struct gl_texture_format _mesa_texformat_rgb888 = {
 const struct gl_texture_format _mesa_texformat_rgb565 = {
    MESA_FORMAT_RGB565,			/* MesaFormat */
    GL_RGB,				/* BaseFormat */
-   GL_UNSIGNED_SHORT_5_6_5,		/* Type */
    5,					/* RedBits */
    6,					/* GreenBits */
    5,					/* BlueBits */
@@ -296,7 +279,6 @@ const struct gl_texture_format _mesa_texformat_rgb565 = {
 const struct gl_texture_format _mesa_texformat_argb4444 = {
    MESA_FORMAT_ARGB4444,		/* MesaFormat */
    GL_RGBA,				/* BaseFormat */
-   GL_UNSIGNED_SHORT_4_4_4_4_REV,	/* Type */
    4,					/* RedBits */
    4,					/* GreenBits */
    4,					/* BlueBits */
@@ -314,7 +296,6 @@ const struct gl_texture_format _mesa_texformat_argb4444 = {
 const struct gl_texture_format _mesa_texformat_argb1555 = {
    MESA_FORMAT_ARGB1555,		/* MesaFormat */
    GL_RGBA,				/* BaseFormat */
-   GL_UNSIGNED_SHORT_1_5_5_5_REV,	/* Type */
    5,					/* RedBits */
    5,					/* GreenBits */
    5,					/* BlueBits */
@@ -332,7 +313,6 @@ const struct gl_texture_format _mesa_texformat_argb1555 = {
 const struct gl_texture_format _mesa_texformat_al88 = {
    MESA_FORMAT_AL88,			/* MesaFormat */
    GL_LUMINANCE_ALPHA,			/* BaseFormat */
-   GL_UNSIGNED_BYTE,			/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -350,7 +330,6 @@ const struct gl_texture_format _mesa_texformat_al88 = {
 const struct gl_texture_format _mesa_texformat_rgb332 = {
    MESA_FORMAT_RGB332,			/* MesaFormat */
    GL_RGB,				/* BaseFormat */
-   GL_UNSIGNED_BYTE_3_3_2,		/* Type */
    3,					/* RedBits */
    3,					/* GreenBits */
    2,					/* BlueBits */
@@ -368,7 +347,6 @@ const struct gl_texture_format _mesa_texformat_rgb332 = {
 const struct gl_texture_format _mesa_texformat_a8 = {
    MESA_FORMAT_A8,			/* MesaFormat */
    GL_ALPHA,				/* BaseFormat */
-   GL_UNSIGNED_BYTE,			/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -386,7 +364,6 @@ const struct gl_texture_format _mesa_texformat_a8 = {
 const struct gl_texture_format _mesa_texformat_l8 = {
    MESA_FORMAT_L8,			/* MesaFormat */
    GL_LUMINANCE,			/* BaseFormat */
-   GL_UNSIGNED_BYTE,			/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -404,7 +381,6 @@ const struct gl_texture_format _mesa_texformat_l8 = {
 const struct gl_texture_format _mesa_texformat_i8 = {
    MESA_FORMAT_I8,			/* MesaFormat */
    GL_INTENSITY,			/* BaseFormat */
-   GL_UNSIGNED_BYTE,			/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -422,7 +398,6 @@ const struct gl_texture_format _mesa_texformat_i8 = {
 const struct gl_texture_format _mesa_texformat_ci8 = {
    MESA_FORMAT_CI8,			/* MesaFormat */
    GL_COLOR_INDEX,			/* BaseFormat */
-   GL_UNSIGNED_BYTE,			/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -437,15 +412,196 @@ const struct gl_texture_format _mesa_texformat_ci8 = {
    fetch_3d_texel_ci8,			/* FetchTexel3D */
 };
 
+const struct gl_texture_format _mesa_texformat_ycbcr = {
+   MESA_FORMAT_YCBCR,			/* MesaFormat */
+   GL_YCBCR_MESA,			/* BaseFormat */
+   0,					/* RedBits */
+   0,					/* GreenBits */
+   0,					/* BlueBits */
+   0,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   2,					/* TexelBytes */
+   fetch_1d_texel_ycbcr,		/* FetchTexel1D */
+   fetch_2d_texel_ycbcr,		/* FetchTexel2D */
+   fetch_3d_texel_ycbcr,		/* FetchTexel3D */
+};
+
+
+const struct gl_texture_format _mesa_texformat_ycbcr_rev = {
+   MESA_FORMAT_YCBCR_REV,		/* MesaFormat */
+   GL_YCBCR_MESA,			/* BaseFormat */
+   0,					/* RedBits */
+   0,					/* GreenBits */
+   0,					/* BlueBits */
+   0,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   2,					/* TexelBytes */
+   fetch_1d_texel_ycbcr_rev,		/* FetchTexel1D */
+   fetch_2d_texel_ycbcr_rev,		/* FetchTexel2D */
+   fetch_3d_texel_ycbcr_rev,		/* FetchTexel3D */
+};
+
+
+/* Big-endian */
+#if 0
+const struct gl_texture_format _mesa_texformat_abgr8888 = {
+   MESA_FORMAT_ABGR8888,		/* MesaFormat */
+   GL_RGBA,				/* BaseFormat */
+   GL_UNSIGNED_INT_8_8_8_8,		/* Type */
+   8,					/* RedBits */
+   8,					/* GreenBits */
+   8,					/* BlueBits */
+   8,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   4,					/* TexelBytes */
+   fetch_1d_texel_abgr8888,		/* FetchTexel1D */
+   fetch_2d_texel_abgr8888,		/* FetchTexel2D */
+   fetch_3d_texel_abgr8888,		/* FetchTexel3D */
+};
+
+const struct gl_texture_format _mesa_texformat_bgra8888 = {
+   MESA_FORMAT_BGRA8888,		/* MesaFormat */
+   GL_RGBA,				/* BaseFormat */
+   GL_UNSIGNED_INT_8_8_8_8,		/* Type */
+   8,					/* RedBits */
+   8,					/* GreenBits */
+   8,					/* BlueBits */
+   8,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   4,					/* TexelBytes */
+   fetch_1d_texel_bgra8888,		/* FetchTexel1D */
+   fetch_2d_texel_bgra8888,		/* FetchTexel2D */
+   fetch_3d_texel_bgra8888,		/* FetchTexel3D */
+};
+
+const struct gl_texture_format _mesa_texformat_bgr888 = {
+   MESA_FORMAT_BGR888,			/* MesaFormat */
+   GL_RGB,				/* BaseFormat */
+   GL_UNSIGNED_BYTE,			/* Type */
+   8,					/* RedBits */
+   8,					/* GreenBits */
+   8,					/* BlueBits */
+   0,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   3,					/* TexelBytes */
+   fetch_1d_texel_bgr888,		/* FetchTexel1D */
+   fetch_2d_texel_bgr888,		/* FetchTexel2D */
+   fetch_3d_texel_bgr888,		/* FetchTexel3D */
+};
+
+const struct gl_texture_format _mesa_texformat_bgr565 = {
+   MESA_FORMAT_BGR565,			/* MesaFormat */
+   GL_RGB,				/* BaseFormat */
+   GL_UNSIGNED_SHORT_5_6_5,		/* Type */
+   5,					/* RedBits */
+   6,					/* GreenBits */
+   5,					/* BlueBits */
+   0,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   2,					/* TexelBytes */
+   fetch_1d_texel_bgr565,		/* FetchTexel1D */
+   fetch_2d_texel_bgr565,		/* FetchTexel2D */
+   fetch_3d_texel_bgr565,		/* FetchTexel3D */
+};
+
+const struct gl_texture_format _mesa_texformat_bgra4444 = {
+   MESA_FORMAT_BGRA4444,		/* MesaFormat */
+   GL_RGBA,				/* BaseFormat */
+   GL_UNSIGNED_SHORT_4_4_4_4_REV,	/* Type */
+   4,					/* RedBits */
+   4,					/* GreenBits */
+   4,					/* BlueBits */
+   4,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   2,					/* TexelBytes */
+   fetch_1d_texel_bgra4444,		/* FetchTexel1D */
+   fetch_2d_texel_bgra4444,		/* FetchTexel2D */
+   fetch_3d_texel_bgra4444,		/* FetchTexel3D */
+};
+
+const struct gl_texture_format _mesa_texformat_bgra5551 = {
+   MESA_FORMAT_BGRA5551,		/* MesaFormat */
+   GL_RGBA,				/* BaseFormat */
+   GL_UNSIGNED_SHORT_1_5_5_5_REV,	/* Type */
+   5,					/* RedBits */
+   5,					/* GreenBits */
+   5,					/* BlueBits */
+   1,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   2,					/* TexelBytes */
+   fetch_1d_texel_bgra1555,		/* FetchTexel1D */
+   fetch_2d_texel_bgra1555,		/* FetchTexel2D */
+   fetch_3d_texel_bgra1555,		/* FetchTexel3D */
+};
+
+const struct gl_texture_format _mesa_texformat_la88 = {
+   MESA_FORMAT_LA88,			/* MesaFormat */
+   GL_LUMINANCE_ALPHA,			/* BaseFormat */
+   GL_UNSIGNED_BYTE,			/* Type */
+   0,					/* RedBits */
+   0,					/* GreenBits */
+   0,					/* BlueBits */
+   8,					/* AlphaBits */
+   8,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   2,					/* TexelBytes */
+   fetch_1d_texel_la88,			/* FetchTexel1D */
+   fetch_2d_texel_la88,			/* FetchTexel2D */
+   fetch_3d_texel_la88,			/* FetchTexel3D */
+};
+
+const struct gl_texture_format _mesa_texformat_bgr233 = {
+   MESA_FORMAT_BGR233,			/* MesaFormat */
+   GL_RGB,				/* BaseFormat */
+   GL_UNSIGNED_BYTE_3_3_2,		/* Type */
+   3,					/* RedBits */
+   3,					/* GreenBits */
+   2,					/* BlueBits */
+   0,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   1,					/* TexelBytes */
+   fetch_1d_texel_bgr233,		/* FetchTexel1D */
+   fetch_2d_texel_bgr233,		/* FetchTexel2D */
+   fetch_3d_texel_bgr233,		/* FetchTexel3D */
+};
+#endif
 
 /* =============================================================
- * Null format:
+ * Null format (useful for proxy textures):
  */
 
 const struct gl_texture_format _mesa_null_texformat = {
    -1,					/* MesaFormat */
    0,					/* BaseFormat */
-   0,					/* Type */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
@@ -461,7 +617,6 @@ const struct gl_texture_format _mesa_null_texformat = {
 };
 
 
-
 GLboolean
 _mesa_is_hardware_tex_format( const struct gl_texture_format *format )
 {
@@ -472,8 +627,8 @@ _mesa_is_hardware_tex_format( const struct gl_texture_format *format )
 /* Given an internal texture format (or 1, 2, 3, 4) return a pointer
  * to a gl_texture_format which which to store the texture.
  * This is called via ctx->Driver.ChooseTextureFormat().
- * Hardware drivers should not use this function, but instead a 
- * specialized function.
+ * Hardware drivers typically override this function with a specialized
+ * version.
  */
 const struct gl_texture_format *
 _mesa_choose_tex_format( GLcontext *ctx, GLint internalFormat,
@@ -590,9 +745,15 @@ _mesa_choose_tex_format( GLcontext *ctx, GLint internalFormat,
 	 _mesa_problem(ctx, "texture compression extension not enabled");
       return &_mesa_texformat_rgba;
 
+   /* GL_MESA_ycrcr_texture */
+   case GL_YCBCR_MESA:
+      if (type == GL_UNSIGNED_SHORT_8_8_MESA)
+         return &_mesa_texformat_ycbcr;
+      else
+         return &_mesa_texformat_ycbcr_rev;
+
    default:
       _mesa_problem(ctx, "unexpected format in _mesa_choose_tex_format()");
-      printf("intformat = %d %x\n", internalFormat, internalFormat);
       return NULL;
    }
 }
@@ -627,23 +788,3 @@ _mesa_base_compressed_texformat(GLcontext *ctx, GLint intFormat)
       return -1;  /* not a recognized compressed format */
    }
 }
-
-
-/*
- * Called via ctx->Driver.CompressedTextureSize().
- * This function is only used by software rasterizers.
- * Hardware drivers will have to implement a specialized function.
- */
-GLint
-_mesa_compressed_texture_size(GLcontext *ctx,
-                              const struct gl_texture_image *texImage)
-{
-   GLint b;
-   assert(texImage);
-   assert(texImage->TexFormat);
-
-   b = texImage->Width * texImage->Height * texImage->Depth *
-      texImage->TexFormat->TexelBytes;
-   assert(b > 0);
-   return b;
-}
