@@ -1,10 +1,10 @@
-/* $Id: texformat.h,v 1.7.2.1 2002/06/08 12:43:40 brianp Exp $ */
+/* $Id: texformat.h,v 1.12 2002/09/27 02:45:38 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.1
  *
- * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -33,17 +33,20 @@
 #include "mtypes.h"
 
 
-/* The Mesa internal texture image types.  These will be set to their
- * default value, but may be changed by drivers as required.
+/*
+ * The Mesa internal texture image types.
+ * All texture images must be stored in one of these formats.
  */
 enum _format {
    /* Hardware-friendly formats.  Drivers can override the default
     * formats and convert texture images to one of these as required.
+    * The driver's ChooseTextureFormat() function will choose one of
+    * these formats.
     * These formats are all little endian, as shown below.  They will be
     * most useful for x86-based PC graphics card drivers.
     *
     * NOTE: In the default case, some of these formats will be
-    * duplicates of the default formats listed above.  However, these
+    * duplicates of the generic formats listed below.  However, these
     * formats guarantee their internal component sizes, while GLchan may
     * vary betwen GLubyte, GLushort and GLfloat.
     */
@@ -61,6 +64,23 @@ enum _format {
    MESA_FORMAT_L8,		/*                               LLLL LLLL */
    MESA_FORMAT_I8,		/*                               IIII IIII */
    MESA_FORMAT_CI8,		/*                               CCCC CCCC */
+   MESA_FORMAT_YCBCR,		/*                     YYYY YYYY UorV UorV */
+   MESA_FORMAT_YCBCR_REV,	/*                     UorV UorV YYYY YYYY */
+
+#if 0
+   /* upcoming little-endian formats: */
+
+				/* msb <------ TEXEL BITS -----------> lsb */
+				/* ---- ---- ---- ---- ---- ---- ---- ---- */
+   MESA_FORMAT_ABGR8888,	/* AAAA AAAA BBBB BBBB GGGG GGGG RRRR RRRR */
+   MESA_FORMAT_BGRA8888,	/* BBBB BBBB GGGG GGGG RRRR RRRR AAAA AAAA */
+   MESA_FORMAT_BGR888,		/*           BBBB BBBB GGGG GGGG RRRR RRRR */
+   MESA_FORMAT_BGR565,		/*                     BBBB BGGG GGGR RRRR */
+   MESA_FORMAT_BGRA4444,	/*                     BBBB GGGG RRRR AAAA */
+   MESA_FORMAT_BGRA5551,	/*                     BBBB BGGG GGRR RRRA */
+   MESA_FORMAT_LA88,		/*                     LLLL LLLL AAAA AAAA */
+   MESA_FORMAT_BGR233,		/*                               BBGG GRRR */
+#endif
 
    /* Generic GLchan-based formats.  These are the default formats used
     * by the software rasterizer and, unless the driver overrides the
@@ -70,7 +90,7 @@ enum _format {
     *
     * NOTE: Because these are based on the GLchan datatype, one cannot
     * assume 8 bits per channel with these formats.  If you require
-    * GLubyte per channel, use one of the hardware formats above.
+    * GLubyte channels, use one of the hardware formats above.
     */
    MESA_FORMAT_RGBA,
    MESA_FORMAT_RGB,
@@ -93,10 +113,6 @@ _mesa_choose_tex_format( GLcontext *ctx, GLint internalFormat,
 extern GLint
 _mesa_base_compressed_texformat(GLcontext *ctx, GLint intFormat);
 
-extern GLint
-_mesa_compressed_texture_size(GLcontext *ctx,
-                              const struct gl_texture_image *texImage);
-
 
 /* The default formats, GLchan per component:
  */
@@ -123,6 +139,8 @@ extern const struct gl_texture_format _mesa_texformat_a8;
 extern const struct gl_texture_format _mesa_texformat_l8;
 extern const struct gl_texture_format _mesa_texformat_i8;
 extern const struct gl_texture_format _mesa_texformat_ci8;
+extern const struct gl_texture_format _mesa_texformat_ycbcr;
+extern const struct gl_texture_format _mesa_texformat_ycbcr_rev;
 
 /* The null format:
  */
