@@ -1,10 +1,10 @@
-/* $Id: t_context.c,v 1.22.2.3 2002/04/09 12:13:08 keithw Exp $ */
+/* $Id: t_context.c,v 1.29 2002/10/24 23:57:25 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.0.3
+ * Version:  3.5
  *
- * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,9 +29,9 @@
 
 
 #include "glheader.h"
+#include "imports.h"
 #include "macros.h"
 #include "mtypes.h"
-#include "mem.h"
 #include "dlist.h"
 #include "light.h"
 #include "vtxfmt.h"
@@ -103,7 +103,7 @@ _tnl_CreateContext( GLcontext *ctx )
    _tnl_install_pipeline( ctx, _tnl_default_pipeline );
 
 
-   tnl->NeedProjCoords = GL_TRUE;
+   tnl->NeedNdcCoords = GL_TRUE;
    tnl->LoopbackDListCassettes = GL_FALSE;
    tnl->CalcDListNormalLengths = GL_TRUE;
 
@@ -194,6 +194,11 @@ _tnl_wakeup_exec( GLcontext *ctx )
     */
    _tnl_InvalidateState( ctx, ~0 );
    tnl->pipeline.run_input_changes = ~0;
+
+   if (ctx->Light.ColorMaterialEnabled) {
+      _mesa_update_color_material( ctx, ctx->Current.Attrib[VERT_ATTRIB_COLOR0] );
+   }
+
 }
 
 
@@ -211,8 +216,8 @@ void
 _tnl_need_projected_coords( GLcontext *ctx, GLboolean mode )
 {
    TNLcontext *tnl = TNL_CONTEXT(ctx);
-   if (tnl->NeedProjCoords != mode) {
-      tnl->NeedProjCoords = mode;
+   if (tnl->NeedNdcCoords != mode) {
+      tnl->NeedNdcCoords = mode;
       _tnl_InvalidateState( ctx, _NEW_PROJECTION );
    }
 }
