@@ -1,10 +1,10 @@
-/* $Id: t_imm_eval.c,v 1.18.2.1 2002/04/09 12:13:09 keithw Exp $ */
+/* $Id: t_imm_eval.c,v 1.26 2002/10/24 23:57:25 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.1
  *
- * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,7 +25,7 @@
  *
  * Authors:
  *    Keith Whitwell <keithw@valinux.com>
- *
+ *    Brian Paul - vertex program updates
  */
 
 
@@ -33,7 +33,7 @@
 #include "colormac.h"
 #include "context.h"
 #include "macros.h"
-#include "mem.h"
+#include "imports.h"
 #include "mmath.h"
 #include "mtypes.h"
 #include "math/m_eval.h"
@@ -52,11 +52,11 @@ static void eval_points1( GLfloat outcoord[][4],
 			  GLfloat du, GLfloat u1 )
 {
    GLuint i;
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & VERT_EVAL_ANY) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & VERT_BITS_EVAL_ANY) {
 	 outcoord[i][0] = coord[i][0];
 	 outcoord[i][1] = coord[i][1];
-	 if (flags[i] & VERT_EVAL_P1)
+	 if (flags[i] & VERT_BIT_EVAL_P1)
 	    outcoord[i][0] = coord[i][0] * du + u1;
       }
 }
@@ -68,11 +68,11 @@ static void eval_points2( GLfloat outcoord[][4],
 			  GLfloat dv, GLfloat v1 )
 {
    GLuint i;
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++) {
-      if (flags[i] & VERT_EVAL_ANY) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++) {
+      if (flags[i] & VERT_BITS_EVAL_ANY) {
 	 outcoord[i][0] = coord[i][0];
 	 outcoord[i][1] = coord[i][1];
-	 if (flags[i] & VERT_EVAL_P2) {
+	 if (flags[i] & VERT_BIT_EVAL_P2) {
 	    outcoord[i][0] = coord[i][0] * du + u1;
 	    outcoord[i][1] = coord[i][1] * dv + v1;
 	 }
@@ -93,15 +93,15 @@ static void eval1_4f( GLvector4f *dest,
 		      GLfloat coord[][4],
 		      const GLuint *flags,
 		      GLuint dimension,
-		      struct gl_1d_map *map )
+		      const struct gl_1d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
    GLfloat (*to)[4] = dest->data;
    GLuint i;
 
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C1|VERT_EVAL_P1)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & (VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 ASSIGN_4V(to[i], 0,0,0,1);
 	 _math_horner_bezier_curve(map->Points, to[i], u,
@@ -112,11 +112,13 @@ static void eval1_4f( GLvector4f *dest,
    dest->flags |= dirty_flags[dimension];
 }
 
+
+/* as above, but dest is a gl_client_array */
 static void eval1_4f_ca( struct gl_client_array *dest,
 			 GLfloat coord[][4],
 			 const GLuint *flags,
 			 GLuint dimension,
-			 struct gl_1d_map *map )
+			 const struct gl_1d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
@@ -126,8 +128,8 @@ static void eval1_4f_ca( struct gl_client_array *dest,
    ASSERT(dest->Type == GL_FLOAT);
    ASSERT(dest->StrideB == 4 * sizeof(GLfloat));
 
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C1|VERT_EVAL_P1)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & (VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 ASSIGN_4V(to[i], 0,0,0,1);
 	 _math_horner_bezier_curve(map->Points, to[i], u,
@@ -141,15 +143,15 @@ static void eval1_4f_ca( struct gl_client_array *dest,
 static void eval1_1ui( GLvector1ui *dest,
 		       GLfloat coord[][4],
 		       const GLuint *flags,
-		       struct gl_1d_map *map )
+		       const struct gl_1d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
    GLuint *to = dest->data;
    GLuint i;
 
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C1|VERT_EVAL_P1)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & (VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 GLfloat tmp;
 	 _math_horner_bezier_curve(map->Points, &tmp, u, 1, map->Order);
@@ -158,46 +160,41 @@ static void eval1_1ui( GLvector1ui *dest,
 
 }
 
-static void eval1_norm( GLvector3f *dest,
+static void eval1_norm( GLvector4f *dest,
 			GLfloat coord[][4],
 			const GLuint *flags,
-			struct gl_1d_map *map )
+			const struct gl_1d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
-   GLfloat (*to)[3] = dest->data;
+   GLfloat (*to)[4] = dest->data;
    GLuint i;
 
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C1|VERT_EVAL_P1)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & (VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 _math_horner_bezier_curve(map->Points, to[i], u, 3, map->Order);
       }
 }
 
 
-
-
-
 static void eval2_obj_norm( GLvector4f *obj_ptr,
-			    GLvector3f *norm_ptr,
+			    GLvector4f *norm_ptr,
 			    GLfloat coord[][4],
 			    GLuint *flags,
 			    GLuint dimension,
-			    struct gl_2d_map *map )
+			    const struct gl_2d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
    const GLfloat v1 = map->v1;
    const GLfloat dv = map->dv;
    GLfloat (*obj)[4] = obj_ptr->data;
-   GLfloat (*normal)[3] = norm_ptr->data;
+   GLfloat (*normal)[4] = norm_ptr->data;
    GLuint i;
 
-/*     fprintf(stderr, "%s\n", __FUNCTION__); */
-
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C2|VERT_EVAL_P2)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & (VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 GLfloat v = (coord[i][1] - v1) * dv;
 	 GLfloat du[4], dv[4];
@@ -229,7 +226,7 @@ static void eval2_4f( GLvector4f *dest,
 		      GLfloat coord[][4],
 		      const GLuint *flags,
 		      GLuint dimension,
-		      struct gl_2d_map *map )
+		      const struct gl_2d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
@@ -238,8 +235,8 @@ static void eval2_4f( GLvector4f *dest,
    GLfloat (*to)[4] = dest->data;
    GLuint i;
 
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C2|VERT_EVAL_P2)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & (VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 GLfloat v = (coord[i][1] - v1) * dv;
 
@@ -251,11 +248,13 @@ static void eval2_4f( GLvector4f *dest,
    dest->flags |= dirty_flags[dimension];
 }
 
+
+/* as above, but dest is a gl_client_array */
 static void eval2_4f_ca( struct gl_client_array *dest,
 			 GLfloat coord[][4],
 			 const GLuint *flags,
 			 GLuint dimension,
-			 struct gl_2d_map *map )
+			 const struct gl_2d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
@@ -267,8 +266,8 @@ static void eval2_4f_ca( struct gl_client_array *dest,
    ASSERT(dest->Type == GL_FLOAT);
    ASSERT(dest->StrideB == 4 * sizeof(GLfloat));
 
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C2|VERT_EVAL_P2)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & (VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 GLfloat v = (coord[i][1] - v1) * dv;
 	 _math_horner_bezier_surf(map->Points, to[i], u, v, dimension,
@@ -279,33 +278,33 @@ static void eval2_4f_ca( struct gl_client_array *dest,
 }
 
 
-static void eval2_norm( GLvector3f *dest,
+static void eval2_norm( GLvector4f *dest,
 			GLfloat coord[][4],
 			GLuint *flags,
-			struct gl_2d_map *map )
+			const struct gl_2d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
    const GLfloat v1 = map->v1;
    const GLfloat dv = map->dv;
-   GLfloat (*to)[3] = dest->data;
+   GLfloat (*to)[4] = dest->data;
    GLuint i;
 
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C2|VERT_EVAL_P2)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++) {
+      if (flags[i] & (VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 GLfloat v = (coord[i][1] - v1) * dv;
 	 _math_horner_bezier_surf(map->Points, to[i], u, v, 3,
 				  map->Uorder, map->Vorder);
-     }
-
+      }
+   }
 }
 
 
 static void eval2_1ui( GLvector1ui *dest,
 		       GLfloat coord[][4],
 		       const GLuint *flags,
-		       struct gl_2d_map *map )
+		       const struct gl_2d_map *map )
 {
    const GLfloat u1 = map->u1;
    const GLfloat du = map->du;
@@ -314,8 +313,8 @@ static void eval2_1ui( GLvector1ui *dest,
    GLuint *to = dest->data;
    GLuint i;
 
-   for (i = 0 ; !(flags[i] & VERT_END_VB) ; i++)
-      if (flags[i] & (VERT_EVAL_C2|VERT_EVAL_P2)) {
+   for (i = 0 ; !(flags[i] & VERT_BIT_END_VB) ; i++)
+      if (flags[i] & (VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2)) {
 	 GLfloat u = (coord[i][0] - u1) * du;
 	 GLfloat v = (coord[i][1] - v1) * dv;
 	 GLfloat tmp;
@@ -327,16 +326,12 @@ static void eval2_1ui( GLvector1ui *dest,
 }
 
 
-
-
-
-
 static void copy_4f( GLfloat to[][4], GLfloat from[][4], GLuint count )
 {
    MEMCPY( to, from, count * sizeof(to[0]));
 }
 
-static void copy_4f_stride( GLfloat to[][4], GLfloat *from, 
+static void copy_4f_stride( GLfloat to[][4], const GLfloat *from, 
 			    GLuint stride, GLuint count )
 {
    if (stride == 4 * sizeof(GLfloat))
@@ -348,7 +343,7 @@ static void copy_4f_stride( GLfloat to[][4], GLfloat *from,
    }
 }
 
-static void copy_3f( GLfloat to[][3], GLfloat from[][3], GLuint count )
+static void copy_3f( GLfloat to[][4], GLfloat from[][4], GLuint count )
 {
    GLuint i;
    for (i = 0 ; i < count ; i++) {
@@ -357,7 +352,7 @@ static void copy_3f( GLfloat to[][3], GLfloat from[][3], GLuint count )
 }
 
 
-static void copy_1ui( GLuint to[], GLuint from[], GLuint count )
+static void copy_1ui( GLuint to[], const GLuint from[], GLuint count )
 {
    MEMCPY( to, from, (count) * sizeof(to[0]));
 }
@@ -370,58 +365,71 @@ static void update_eval( GLcontext *ctx )
 {
    TNLcontext *tnl = TNL_CONTEXT(ctx);
    GLuint eval1 = 0, eval2 = 0;
+   GLuint i;
 
    if (ctx->Eval.Map1Index)
-      eval1 |= VERT_INDEX;
+      eval1 |= VERT_BIT_INDEX;
 
    if (ctx->Eval.Map2Index)
-      eval2 |= VERT_INDEX;
+      eval2 |= VERT_BIT_INDEX;
 
    if (ctx->Eval.Map1Color4)
-      eval1 |= VERT_RGBA;
+      eval1 |= VERT_BIT_COLOR0;
 
    if (ctx->Eval.Map2Color4)
-      eval2 |= VERT_RGBA;
+      eval2 |= VERT_BIT_COLOR0;
 
    if (ctx->Eval.Map1Normal)
-      eval1 |= VERT_NORM;
+      eval1 |= VERT_BIT_NORMAL;
 
    if (ctx->Eval.Map2Normal)
-      eval2 |= VERT_NORM;
+      eval2 |= VERT_BIT_NORMAL;
 
    if (ctx->Eval.Map1TextureCoord4 ||
        ctx->Eval.Map1TextureCoord3 ||
        ctx->Eval.Map1TextureCoord2 ||
        ctx->Eval.Map1TextureCoord1)
-      eval1 |= VERT_TEX0;
+      eval1 |= VERT_BIT_TEX0;
 
    if (ctx->Eval.Map2TextureCoord4 ||
        ctx->Eval.Map2TextureCoord3 ||
        ctx->Eval.Map2TextureCoord2 ||
        ctx->Eval.Map2TextureCoord1)
-      eval2 |= VERT_TEX0;
+      eval2 |= VERT_BIT_TEX0;
 
    if (ctx->Eval.Map1Vertex4)
-      eval1 |= VERT_OBJ_234;
+      eval1 |= VERT_BITS_OBJ_234;
 
    if (ctx->Eval.Map1Vertex3)
-      eval1 |= VERT_OBJ_23;
+      eval1 |= VERT_BITS_OBJ_23;
 
    if (ctx->Eval.Map2Vertex4) {
       if (ctx->Eval.AutoNormal)
-	 eval2 |= VERT_OBJ_234 | VERT_NORM;
+	 eval2 |= VERT_BITS_OBJ_234 | VERT_BIT_NORMAL;
       else
-	 eval2 |= VERT_OBJ_234;
+	 eval2 |= VERT_BITS_OBJ_234;
    }
    else if (ctx->Eval.Map2Vertex3) {
       if (ctx->Eval.AutoNormal)
-	 eval2 |= VERT_OBJ_23 | VERT_NORM;
+	 eval2 |= VERT_BITS_OBJ_23 | VERT_BIT_NORMAL;
       else
-	 eval2 |= VERT_OBJ_23;
+	 eval2 |= VERT_BITS_OBJ_23;
    }
 
    tnl->eval.EvalMap1Flags = eval1;
    tnl->eval.EvalMap2Flags = eval2;
+
+   /* GL_NV_vertex_program evaluators */
+   eval1 = eval2 = 0;
+   for (i = 0; i < VERT_ATTRIB_MAX; i++) {
+      if (ctx->Eval.Map1Attrib[i])
+         eval1 |= (1 << i);
+      if (ctx->Eval.Map2Attrib[i])
+         eval2 |= (1 << i);
+   }
+   tnl->eval.EvalMap1AttribFlags = eval1;
+   tnl->eval.EvalMap2AttribFlags = eval2;
+
    tnl->eval.EvalNewState = 0;
 }
 
@@ -441,13 +449,13 @@ void _tnl_eval_immediate( GLcontext *ctx, struct immediate *IM )
    GLuint *flags = IM->Flag + IM->CopyStart;
    GLuint copycount;
    GLuint orflag = IM->OrFlag;
-   GLuint any_eval1 = orflag & (VERT_EVAL_C1|VERT_EVAL_P1);
-   GLuint any_eval2 = orflag & (VERT_EVAL_C2|VERT_EVAL_P2);
+   GLuint any_eval1 = orflag & (VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1);
+   GLuint any_eval2 = orflag & (VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2);
    GLuint req = 0;
    GLuint purge_flags = 0;
-   GLfloat (*coord)[4] = IM->Obj + IM->CopyStart;
+   GLfloat (*coord)[4] = IM->Attrib[VERT_ATTRIB_POS] + IM->CopyStart;
 
-   if (IM->AndFlag & VERT_EVAL_ANY)
+   if (IM->AndFlag & VERT_BITS_EVAL_ANY)
       copycount = IM->Start - IM->CopyStart; /* just copy copied vertices */
    else
       copycount = IM->Count - IM->CopyStart; /* copy all vertices */
@@ -459,46 +467,46 @@ void _tnl_eval_immediate( GLcontext *ctx, struct immediate *IM )
       update_eval( ctx );
 
    if (any_eval1) {
-      req |= tnl->pipeline.inputs & tnl->eval.EvalMap1Flags;
+      req |= tnl->pipeline.inputs
+         & (tnl->eval.EvalMap1Flags | tnl->eval.EvalMap1AttribFlags);
 
-      if (!ctx->Eval.Map1Vertex4 && !ctx->Eval.Map1Vertex3)
-	 purge_flags = (VERT_EVAL_P1|VERT_EVAL_C1);
+      if (!ctx->Eval.Map1Vertex4 && !ctx->Eval.Map1Vertex3 &&
+          !ctx->Eval.Map1Attrib[0])
+	 purge_flags = (VERT_BIT_EVAL_P1|VERT_BIT_EVAL_C1);
 
-      if (orflag & VERT_EVAL_P1) {
-	 eval_points1( store->Obj + IM->CopyStart, 
+      if (orflag & VERT_BIT_EVAL_P1) {
+	 eval_points1( store->Attrib[VERT_ATTRIB_POS] + IM->CopyStart, 
 		       coord, flags,
 		       ctx->Eval.MapGrid1du,
 		       ctx->Eval.MapGrid1u1);
 	 
-	 coord = store->Obj + IM->CopyStart;
+	 coord = store->Attrib[VERT_ATTRIB_POS] + IM->CopyStart;
       }
    }
 
    if (any_eval2) {
-      req |= tnl->pipeline.inputs & tnl->eval.EvalMap2Flags;
+      req |= tnl->pipeline.inputs
+         & (tnl->eval.EvalMap2Flags | tnl->eval.EvalMap2AttribFlags);
 
-      if (!ctx->Eval.Map2Vertex4 && !ctx->Eval.Map2Vertex3)
-	 purge_flags |= (VERT_EVAL_P2|VERT_EVAL_C2);
+      if (!ctx->Eval.Map2Vertex4 && !ctx->Eval.Map2Vertex3 &&
+          !ctx->Eval.Map2Attrib[0])
+	 purge_flags |= (VERT_BIT_EVAL_P2|VERT_BIT_EVAL_C2);
 
-      if (orflag & VERT_EVAL_P2) {
-	 eval_points2( store->Obj + IM->CopyStart, 
+      if (orflag & VERT_BIT_EVAL_P2) {
+	 eval_points2( store->Attrib[VERT_ATTRIB_POS] + IM->CopyStart, 
 		       coord, flags,
 		       ctx->Eval.MapGrid2du,
 		       ctx->Eval.MapGrid2u1,
 		       ctx->Eval.MapGrid2dv,
 		       ctx->Eval.MapGrid2v1 );
 
-	 coord = store->Obj + IM->CopyStart;
+	 coord = store->Attrib[VERT_ATTRIB_POS] + IM->CopyStart;
       }
    }
 
-
-/*     _tnl_print_vert_flags(__FUNCTION__, req); */
-
    /* Perform the evaluations on active data elements.
     */
-   if (req & VERT_INDEX)
-   {
+   if (req & VERT_BIT_INDEX) {
       GLuint generated = 0;
 
       if (copycount)
@@ -509,152 +517,230 @@ void _tnl_eval_immediate( GLcontext *ctx, struct immediate *IM )
 
       if (ctx->Eval.Map1Index && any_eval1) {
 	 eval1_1ui( &tmp->Index, coord, flags, &ctx->EvalMap.Map1Index );
-	 generated |= VERT_EVAL_C1|VERT_EVAL_P1;
+	 generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
       }
 
       if (ctx->Eval.Map2Index && any_eval2) {
 	 eval2_1ui( &tmp->Index, coord, flags, &ctx->EvalMap.Map2Index );
-	 generated |= VERT_EVAL_C2|VERT_EVAL_P2;
+	 generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
       }
    }
 
-   if (req & VERT_RGBA)
-   {
+   if (req & VERT_BIT_COLOR0) {
       GLuint generated = 0;
 
       if (copycount) 
-	 copy_4f_stride( store->Color + IM->CopyStart, 
+	 copy_4f_stride( store->Attrib[VERT_ATTRIB_COLOR0] + IM->CopyStart, 
 			 (GLfloat *)tmp->Color.Ptr, 
 			 tmp->Color.StrideB,
 			 copycount );
 
-      tmp->Color.Ptr = store->Color + IM->CopyStart;
+      tmp->Color.Ptr = store->Attrib[VERT_ATTRIB_COLOR0] + IM->CopyStart;
       tmp->Color.StrideB = 4 * sizeof(GLfloat);
       tmp->Color.Flags = 0;
-      tnl->vb.importable_data &= ~VERT_RGBA;
+      tnl->vb.importable_data &= ~VERT_BIT_COLOR0;
 
-      if (ctx->Eval.Map1Color4 && any_eval1) {
-	 eval1_4f_ca( &tmp->Color, coord, flags, 4, &ctx->EvalMap.Map1Color4 );
-	 generated |= VERT_EVAL_C1|VERT_EVAL_P1;
+      if (ctx->VertexProgram.Enabled) {
+         tmp->Attribs[VERT_ATTRIB_COLOR0].data =
+            store->Attrib[VERT_ATTRIB_COLOR0] + IM->CopyStart;
+         tmp->Attribs[VERT_ATTRIB_COLOR0].start =
+            (GLfloat *) tmp->Attribs[VERT_ATTRIB_COLOR0].data;
+         tmp->Attribs[VERT_ATTRIB_COLOR0].size = 0;
       }
 
-      if (ctx->Eval.Map2Color4 && any_eval2) {
-	 eval2_4f_ca( &tmp->Color, coord, flags, 4, &ctx->EvalMap.Map2Color4 );
-	 generated |= VERT_EVAL_C2|VERT_EVAL_P2;
+      /* Vertex program maps have priority over conventional attribs */
+      if (any_eval1) {
+         if (ctx->VertexProgram.Enabled
+             && ctx->Eval.Map1Attrib[VERT_ATTRIB_COLOR0]) {
+            eval1_4f_ca( &tmp->Color, coord, flags, 4,
+                         &ctx->EvalMap.Map1Attrib[VERT_ATTRIB_COLOR0] );
+            generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
+         }
+         else if (ctx->Eval.Map1Color4) {
+            eval1_4f_ca( &tmp->Color, coord, flags, 4,
+                         &ctx->EvalMap.Map1Color4 );
+            generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
+         }
+      }
+
+      if (any_eval2) {
+         if (ctx->VertexProgram.Enabled
+             && ctx->Eval.Map2Attrib[VERT_ATTRIB_COLOR0]) {
+            eval2_4f_ca( &tmp->Color, coord, flags, 4,
+                         &ctx->EvalMap.Map2Attrib[VERT_ATTRIB_COLOR0] );
+            generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
+         }
+         else if (ctx->Eval.Map2Color4) {
+            eval2_4f_ca( &tmp->Color, coord, flags, 4,
+                         &ctx->EvalMap.Map2Color4 );
+            generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
+         }
       }
    }
 
-
-   if (req & VERT_TEX(0))
-   {
+   if (req & VERT_BIT_TEX0) {
       GLuint generated = 0;
 
       if (copycount)
-	 copy_4f( store->TexCoord[0] + IM->CopyStart, 
+	 copy_4f( store->Attrib[VERT_ATTRIB_TEX0] + IM->CopyStart, 
 		  tmp->TexCoord[0].data, copycount );
       else
 	 tmp->TexCoord[0].size = 0;
 
-      tmp->TexCoord[0].data = store->TexCoord[0] + IM->CopyStart;
+      tmp->TexCoord[0].data = store->Attrib[VERT_ATTRIB_TEX0] + IM->CopyStart;
       tmp->TexCoord[0].start = (GLfloat *)tmp->TexCoord[0].data;
 
+      if (ctx->VertexProgram.Enabled) {
+         tmp->Attribs[VERT_ATTRIB_TEX0].data =
+            store->Attrib[VERT_ATTRIB_TEX0] + IM->CopyStart;
+         tmp->Attribs[VERT_ATTRIB_TEX0].start =
+            (GLfloat *) tmp->Attribs[VERT_ATTRIB_TEX0].data;
+         tmp->Attribs[VERT_ATTRIB_TEX0].size = 0;
+      }
+
+      /* Vertex program maps have priority over conventional attribs */
       if (any_eval1) {
-	 if (ctx->Eval.Map1TextureCoord4) {
+         if (ctx->VertexProgram.Enabled
+             && ctx->Eval.Map1Attrib[VERT_ATTRIB_TEX0]) {
+	    eval1_4f( &tmp->TexCoord[0], coord, flags, 4,
+		      &ctx->EvalMap.Map1Attrib[VERT_ATTRIB_TEX0] );
+	    generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
+         }
+	 else if (ctx->Eval.Map1TextureCoord4) {
 	    eval1_4f( &tmp->TexCoord[0], coord, flags, 4,
 		      &ctx->EvalMap.Map1Texture4 );
-	    generated |= VERT_EVAL_C1|VERT_EVAL_P1;
+	    generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
 	 }
 	 else if (ctx->Eval.Map1TextureCoord3) {
 	    eval1_4f( &tmp->TexCoord[0], coord, flags, 3,
 		      &ctx->EvalMap.Map1Texture3 );
-	    generated |= VERT_EVAL_C1|VERT_EVAL_P1;
+	    generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
 	 }
 	 else if (ctx->Eval.Map1TextureCoord2) {
 	    eval1_4f( &tmp->TexCoord[0], coord, flags, 2,
 		      &ctx->EvalMap.Map1Texture2 );
-	    generated |= VERT_EVAL_C1|VERT_EVAL_P1;
+	    generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
 	 }
 	 else if (ctx->Eval.Map1TextureCoord1) {
 	    eval1_4f( &tmp->TexCoord[0], coord, flags, 1,
 		      &ctx->EvalMap.Map1Texture1 );
-	    generated |= VERT_EVAL_C1|VERT_EVAL_P1;
+	    generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
 	 }
       }
 
       if (any_eval2) {
-	 if (ctx->Eval.Map2TextureCoord4) {
+         if (ctx->VertexProgram.Enabled
+             && ctx->Eval.Map2Attrib[VERT_ATTRIB_TEX0]) {
+	    eval2_4f( &tmp->TexCoord[0], coord, flags, 4,
+		      &ctx->EvalMap.Map2Attrib[VERT_ATTRIB_TEX0] );
+	    generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
+         }
+	 else if (ctx->Eval.Map2TextureCoord4) {
 	    eval2_4f( &tmp->TexCoord[0], coord, flags, 4,
 		      &ctx->EvalMap.Map2Texture4 );
-	    generated |= VERT_EVAL_C2|VERT_EVAL_P2;
+	    generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
 	 }
 	 else if (ctx->Eval.Map2TextureCoord3) {
 	    eval2_4f( &tmp->TexCoord[0], coord, flags, 3,
 		      &ctx->EvalMap.Map2Texture3 );
-	    generated |= VERT_EVAL_C2|VERT_EVAL_P2;
+	    generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
 	 }
 	 else if (ctx->Eval.Map2TextureCoord2) {
 	    eval2_4f( &tmp->TexCoord[0], coord, flags, 2,
 		      &ctx->EvalMap.Map2Texture2 );
-	    generated |= VERT_EVAL_C2|VERT_EVAL_P2;
+	    generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
 	 }
 	 else if (ctx->Eval.Map2TextureCoord1) {
 	    eval2_4f( &tmp->TexCoord[0], coord, flags, 1,
 		      &ctx->EvalMap.Map2Texture1 );
-	    generated |= VERT_EVAL_C2|VERT_EVAL_P2;
+	    generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
 	 }
       }
    }
 
-
-   if (req & VERT_NORM)
-   {
+   if (req & VERT_BIT_NORMAL) {
       GLuint generated = 0;
 
       if (copycount) {
-	 copy_3f( store->Normal + IM->CopyStart, tmp->Normal.data, 
-		  copycount );
+	 copy_3f( store->Attrib[VERT_ATTRIB_NORMAL] + IM->CopyStart,
+                  tmp->Normal.data, copycount );
       }
 
-      tmp->Normal.data = store->Normal + IM->CopyStart;
+      tmp->Normal.data = store->Attrib[VERT_ATTRIB_NORMAL] + IM->CopyStart;
       tmp->Normal.start = (GLfloat *)tmp->Normal.data;
 
-      if (ctx->Eval.Map1Normal && any_eval1) {
-	 eval1_norm( &tmp->Normal, coord, flags,
-		     &ctx->EvalMap.Map1Normal );
-	 generated |= VERT_EVAL_C1|VERT_EVAL_P1;
+      if (ctx->VertexProgram.Enabled) {
+         tmp->Attribs[VERT_ATTRIB_NORMAL].data =
+            store->Attrib[VERT_ATTRIB_NORMAL] + IM->CopyStart;
+         tmp->Attribs[VERT_ATTRIB_NORMAL].start =
+            (GLfloat *) tmp->Attribs[VERT_ATTRIB_NORMAL].data;
+         tmp->Attribs[VERT_ATTRIB_NORMAL].size = 0;
       }
 
-      if (ctx->Eval.Map2Normal && any_eval2) {
-	 eval2_norm( &tmp->Normal, coord, flags,
-		     &ctx->EvalMap.Map2Normal );
-	 generated |= VERT_EVAL_C2|VERT_EVAL_P2;
+      if (any_eval1) {
+         if (ctx->VertexProgram.Enabled &&
+             ctx->Eval.Map1Attrib[VERT_ATTRIB_NORMAL]) {
+            eval1_norm( &tmp->Normal, coord, flags,
+                        &ctx->EvalMap.Map1Attrib[VERT_ATTRIB_NORMAL] );
+            generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
+         }
+         else if (ctx->Eval.Map1Normal) {
+            eval1_norm( &tmp->Normal, coord, flags, &ctx->EvalMap.Map1Normal );
+            generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
+         }
+      }
+
+      if (any_eval2) {
+         if (ctx->VertexProgram.Enabled &&
+             ctx->Eval.Map2Attrib[VERT_ATTRIB_NORMAL]) {
+            eval2_norm( &tmp->Normal, coord, flags,
+                        &ctx->EvalMap.Map2Attrib[VERT_ATTRIB_NORMAL] );
+            generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
+         }
+         else if (ctx->Eval.Map2Normal) {
+            eval2_norm( &tmp->Normal, coord, flags, &ctx->EvalMap.Map2Normal );
+            generated |= VERT_BIT_EVAL_C2|VERT_BIT_EVAL_P2;
+         }
       }
    }
 
-
-
    /* In the AutoNormal case, the copy and assignment of tmp->NormalPtr
     * are done above.
     */
-   if (req & VERT_OBJ)
-   {
+   if (req & VERT_BIT_POS) {
       if (copycount) {
 	 /* This copy may already have occurred when eliminating
 	  * glEvalPoint calls:
 	  */
-	 if  (coord != store->Obj + IM->CopyStart)
-	    copy_4f( store->Obj + IM->CopyStart, tmp->Obj.data, copycount );
-      } else
+	 if (coord != store->Attrib[VERT_ATTRIB_POS] + IM->CopyStart) {
+	    copy_4f( store->Attrib[VERT_ATTRIB_POS] + IM->CopyStart,
+                     tmp->Obj.data, copycount );
+         }
+      }
+      else {
 	 tmp->Obj.size = 0;
+      }
 
-      tmp->Obj.data = store->Obj + IM->CopyStart;
-      tmp->Obj.start = (GLfloat *)tmp->Obj.data;
+      tmp->Obj.data = store->Attrib[VERT_ATTRIB_POS] + IM->CopyStart;
+      tmp->Obj.start = (GLfloat *) tmp->Obj.data;
+
+#if 1
+      /*tmp->Attribs[0].count = count;*/
+      tmp->Attribs[0].data = store->Attrib[0] + IM->CopyStart;
+      tmp->Attribs[0].start = (GLfloat *) tmp->Attribs[0].data;
+      tmp->Attribs[0].size = 0;
+#endif
 
       /* Note: Normal data is already prepared above.
        */
 
       if (any_eval1) {
-	 if (ctx->Eval.Map1Vertex4) {
+         if (ctx->VertexProgram.Enabled &&
+             ctx->Eval.Map1Attrib[VERT_ATTRIB_POS]) {
+	    eval1_4f( &tmp->Obj, coord, flags, 4,
+		      &ctx->EvalMap.Map1Attrib[VERT_ATTRIB_POS] );
+         }
+	 else if (ctx->Eval.Map1Vertex4) {
 	    eval1_4f( &tmp->Obj, coord, flags, 4,
 		      &ctx->EvalMap.Map1Vertex4 );
 	 }
@@ -665,18 +751,25 @@ void _tnl_eval_immediate( GLcontext *ctx, struct immediate *IM )
       }
 
       if (any_eval2) {
-	 if (ctx->Eval.Map2Vertex4)
-	 {
-	    if (ctx->Eval.AutoNormal && (req & VERT_NORM))
+         if (ctx->VertexProgram.Enabled &&
+             ctx->Eval.Map2Attrib[VERT_ATTRIB_POS]) {
+	    if (ctx->Eval.AutoNormal && (req & VERT_BIT_NORMAL))
+	       eval2_obj_norm( &tmp->Obj, &tmp->Normal, coord, flags, 4,
+			       &ctx->EvalMap.Map2Attrib[VERT_ATTRIB_POS] );
+	    else
+	       eval2_4f( &tmp->Obj, coord, flags, 4,
+			 &ctx->EvalMap.Map2Attrib[VERT_ATTRIB_POS] );
+         }
+	 else if (ctx->Eval.Map2Vertex4) {
+	    if (ctx->Eval.AutoNormal && (req & VERT_BIT_NORMAL))
 	       eval2_obj_norm( &tmp->Obj, &tmp->Normal, coord, flags, 4,
 			       &ctx->EvalMap.Map2Vertex4 );
 	    else
 	       eval2_4f( &tmp->Obj, coord, flags, 4,
 			 &ctx->EvalMap.Map2Vertex4 );
 	 }
-	 else if (ctx->Eval.Map2Vertex3)
-	 {
-	    if (ctx->Eval.AutoNormal && (req & VERT_NORM))
+	 else if (ctx->Eval.Map2Vertex3) {
+	    if (ctx->Eval.AutoNormal && (req & VERT_BIT_NORMAL))
 	       eval2_obj_norm( &tmp->Obj, &tmp->Normal, coord, flags, 3,
 			       &ctx->EvalMap.Map2Vertex3 );
 	    else
@@ -687,18 +780,47 @@ void _tnl_eval_immediate( GLcontext *ctx, struct immediate *IM )
    }
 
 
+   if (ctx->VertexProgram.Enabled) {
+      /* We already evaluated position, normal, color and texture 0 above.
+       * now evaluate any other generic attributes.
+       */
+      const GLuint skipBits = (VERT_BIT_POS |
+                               VERT_BIT_NORMAL |
+                               VERT_BIT_COLOR0 |
+                               VERT_BIT_TEX0);
+      GLuint generated = 0;
+      GLuint attr;
+      for (attr = 0; attr < VERT_ATTRIB_MAX; attr++) {
+         if ((1 << attr) & req & ~skipBits) {
+            if (any_eval1 && ctx->Eval.Map1Attrib[attr]) {
+               /* evaluate 1-D vertex attrib map [i] */
+               eval1_4f( &tmp->Attribs[attr], coord, flags, 4,
+                         &ctx->EvalMap.Map1Attrib[attr] );
+               generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
+            }
+            if (any_eval2 && ctx->Eval.Map2Attrib[attr]) {
+               /* evaluate 2-D vertex attrib map [i] */
+               eval2_4f( &tmp->Attribs[attr], coord, flags, 4,
+                         &ctx->EvalMap.Map2Attrib[attr] );
+               generated |= VERT_BIT_EVAL_C1|VERT_BIT_EVAL_P1;
+            }
+         }
+      }
+   }
+
    /* Calculate new IM->Elts, IM->Primitive, IM->PrimitiveLength for
     * the case where vertex maps are not enabled for some received
     * eval coordinates.  In this case those slots in the immediate
     * must be ignored.
     */
    if (purge_flags) {
-      GLuint vertex = VERT_OBJ|(VERT_EVAL_ANY & ~purge_flags);
+      const GLuint vertex = VERT_BIT_POS|(VERT_BITS_EVAL_ANY & ~purge_flags);
       GLuint last_new_prim = 0;
       GLuint new_prim_length = 0;
       GLuint next_old_prim = 0;
       struct vertex_buffer *VB = &tnl->vb;
-      GLuint i,j,count = VB->Count;
+      const GLuint count = VB->Count;
+      GLuint i, j;
 
       for (i = 0, j = 0 ; i < count ; i++) {
 	 if (flags[i] & vertex) {
@@ -720,8 +842,8 @@ void _tnl_eval_immediate( GLcontext *ctx, struct immediate *IM )
    /* Produce new flags array:
     */
    {
+      const GLuint count = tnl->vb.Count + 1;
       GLuint i;
-      GLuint count = tnl->vb.Count + 1;
 
       copy_1ui( store->Flag, flags, count );
       tnl->vb.Flag = store->Flag;
