@@ -1,4 +1,4 @@
-/* $Id: t_vb_vertex.c,v 1.9.2.1 2002/04/09 12:13:09 keithw Exp $ */
+/* $Id: t_vb_vertex.c,v 1.15 2002/10/24 23:57:25 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -32,7 +32,7 @@
 #include "colormac.h"
 #include "context.h"
 #include "macros.h"
-#include "mem.h"
+#include "imports.h"
 #include "mmath.h"
 #include "mtypes.h"
 
@@ -57,7 +57,7 @@ struct vertex_stage_data {
     */
    GLvector4f *save_eyeptr;
    GLvector4f *save_clipptr;
-   GLvector4f *save_projptr;
+   GLvector4f *save_ndcptr;
 };
 
 #define VERTEX_STAGE_DATA(stage) ((struct vertex_stage_data *)stage->privatePtr)
@@ -79,7 +79,7 @@ static void NAME( GLcontext *ctx,				\
    GLuint p;							\
 								\
    for (p = 0; p < ctx->Const.MaxClipPlanes; p++)		\
-      if (ctx->Transform.ClipEnabled[p]) {			\
+      if (ctx->Transform.ClipPlanesEnabled & (1 << p)) {	\
 	 GLuint nr, i;						\
 	 const GLfloat a = ctx->Transform._ClipUserPlane[p][0];	\
 	 const GLfloat b = ctx->Transform._ClipUserPlane[p][1];	\
@@ -143,16 +143,18 @@ static GLboolean run_vertex_stage( GLcontext *ctx,
 	 /* Separate modelview transformation:
 	  * Use combined ModelProject to avoid some depth artifacts
 	  */
-	 if (ctx->ModelView.type == MATRIX_IDENTITY)
+	 if (ctx->ModelviewMatrixStack.Top->type == MATRIX_IDENTITY)
 	    VB->EyePtr = VB->ObjPtr;
 	 else
-	    VB->EyePtr = TransformRaw( &store->eye, &ctx->ModelView,
+	    VB->EyePtr = TransformRaw( &store->eye,
+                                       ctx->ModelviewMatrixStack.Top,
 				       VB->ObjPtr);
 
-	 if (ctx->ProjectionMatrix.type == MATRIX_IDENTITY)
+	 if (ctx->ProjectionMatrixStack.Top->type == MATRIX_IDENTITY)
 	    VB->ClipPtr = VB->EyePtr;
 	 else
-	    VB->ClipPtr = TransformRaw( &store->clip, &ctx->_ModelProjectMatrix,
+	    VB->ClipPtr = TransformRaw( &store->clip,
+                                        &ctx->_ModelProjectMatrix,
 					VB->ObjPtr );
       }
       else {
@@ -171,7 +173,7 @@ static GLboolean run_vertex_stage( GLcontext *ctx,
       if (VB->ClipPtr->size < 4) {
 	 if (VB->ClipPtr->flags & VEC_NOT_WRITEABLE) {
 	    ASSERT(VB->ClipPtr == VB->ObjPtr);
-	    VB->import_data( ctx, VERT_OBJ, VEC_NOT_WRITEABLE );
+	    VB->import_data( ctx, VERT_BIT_POS, VEC_NOT_WRITEABLE );
 	    VB->ClipPtr = VB->ObjPtr;
 	 }
 	 if (VB->ClipPtr->size == 2)
@@ -185,16 +187,17 @@ static GLboolean run_vertex_stage( GLcontext *ctx,
       store->ormask = 0;
       store->andmask = CLIP_ALL_BITS;
 
-      if (tnl->NeedProjCoords) {
-	 VB->ProjectedClipPtr =
+      if (tnl->NeedNdcCoords) {
+	 VB->NdcPtr =
 	    _mesa_clip_tab[VB->ClipPtr->size]( VB->ClipPtr,
                                                &store->proj,
                                                store->clipmask,
                                                &store->ormask,
                                                &store->andmask );
 
-      } else {
-	 VB->ProjectedClipPtr = 0;
+      }
+      else {
+	 VB->NdcPtr = 0;
 	 _mesa_clip_np_tab[VB->ClipPtr->size]( VB->ClipPtr,
                                                0,
                                                store->clipmask,
@@ -209,7 +212,7 @@ static GLboolean run_vertex_stage( GLcontext *ctx,
       /* Test userclip planes.  This contributes to VB->ClipMask, so
        * is essentially required to be in this stage.
        */
-      if (ctx->Transform._AnyClip) {
+      if (ctx->Transform.ClipPlanesEnabled) {
 	 usercliptab[VB->ClipPtr->size]( ctx,
 					 VB->ClipPtr,
 					 store->clipmask,
@@ -223,23 +226,23 @@ static GLboolean run_vertex_stage( GLcontext *ctx,
       VB->ClipOrMask = store->ormask;
       VB->ClipMask = store->clipmask;
 
-      if (VB->ClipPtr == VB->ObjPtr && (VB->importable_data & VERT_OBJ))
-	 VB->importable_data |= VERT_CLIP;
+      if (VB->ClipPtr == VB->ObjPtr && (VB->importable_data & VERT_BIT_POS))
+	 VB->importable_data |= VERT_BIT_CLIP;
 
       store->save_eyeptr = VB->EyePtr;
       store->save_clipptr = VB->ClipPtr;
-      store->save_projptr = VB->ProjectedClipPtr;
+      store->save_ndcptr = VB->NdcPtr;
    }
    else {
       /* Replay the sideeffects.
        */
       VB->EyePtr = store->save_eyeptr;
       VB->ClipPtr = store->save_clipptr;
-      VB->ProjectedClipPtr = store->save_projptr;
+      VB->NdcPtr = store->save_ndcptr;
       VB->ClipMask = store->clipmask;
       VB->ClipOrMask = store->ormask;
-      if (VB->ClipPtr == VB->ObjPtr && (VB->importable_data & VERT_OBJ))
-	 VB->importable_data |= VERT_CLIP;
+      if (VB->ClipPtr == VB->ObjPtr && (VB->importable_data & VERT_BIT_POS))
+	 VB->importable_data |= VERT_BIT_CLIP;
       if (store->andmask)
 	 return GL_FALSE;
    }
@@ -250,8 +253,7 @@ static GLboolean run_vertex_stage( GLcontext *ctx,
 
 static void check_vertex( GLcontext *ctx, struct gl_pipeline_stage *stage )
 {
-   (void) ctx;
-   (void) stage;
+   stage->active = !ctx->VertexProgram.Enabled;
 }
 
 static GLboolean init_vertex_stage( GLcontext *ctx,
@@ -309,9 +311,10 @@ const struct gl_pipeline_stage _tnl_vertex_transform_stage =
    _NEW_PROJECTION|
    _NEW_TRANSFORM,		/* re-run */
    GL_TRUE,			/* active */
-   VERT_OBJ,		/* inputs */
-   VERT_EYE|VERT_CLIP,		/* outputs */
-   0, 0,			/* changed_inputs, private */
+   VERT_BIT_POS,		/* inputs */
+   VERT_BIT_EYE|VERT_BIT_CLIP,		/* outputs */
+   0,				/* changed_inputs */
+   NULL,			/* private data */
    dtr,				/* destructor */
    check_vertex,		/* check */
    init_vertex_stage		/* run -- initially set to init */
