@@ -1,8 +1,8 @@
-/* $Id: t_dd_vbtmp.h,v 1.14.2.4 2002/06/14 03:49:10 brianp Exp $ */
+/* $Id: t_dd_vbtmp.h,v 1.22 2002/10/23 10:24:27 keithw Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.0.3
+ * Version:  4.1
  *
  * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
@@ -61,7 +61,10 @@
  *    unsigned char ub4[4][16];
  * }
  *
-
+ 
+ * VERTEX:   hw vertex type as above
+ * VERTEX_COLOR: hw color struct type in VERTEX
+ *
  * DO_XYZW:  Emit xyz and maybe w coordinates.
  * DO_RGBA:  Emit color.
  * DO_SPEC:  Emit specular color.
@@ -122,7 +125,7 @@ static void TAG(emit)( GLcontext *ctx,
 {
    LOCALVARS
    struct vertex_buffer *VB = &TNL_CONTEXT(ctx)->vb;
-   GLfloat (*tc0)[4], (*tc1)[4], *fog;
+   GLfloat (*tc0)[4], (*tc1)[4], (*fog)[4];
    GLfloat (*tc2)[4], (*tc3)[4];
    GLubyte (*col)[4], (*spec)[4];
    GLuint tc0_stride, tc1_stride, col_stride, spec_stride, fog_stride;
@@ -145,8 +148,8 @@ static void TAG(emit)( GLcontext *ctx,
       coord_stride = VB->ClipPtr->stride;
    }
    else {
-      coord = VB->ProjectedClipPtr->data;
-      coord_stride = VB->ProjectedClipPtr->stride;
+      coord = VB->NdcPtr->data;
+      coord_stride = VB->NdcPtr->stride;
    }
 
    if (DO_TEX3) {
@@ -207,7 +210,7 @@ static void TAG(emit)( GLcontext *ctx,
 	 fog_stride = VB->FogCoordPtr->stride;
       }
       else {
-	 GLfloat tmp = 0;
+	 static GLfloat tmp[4] = {0, 0, 0, 0};
 	 fog = &tmp;
 	 fog_stride = 0;
       }
@@ -231,7 +234,8 @@ static void TAG(emit)( GLcontext *ctx,
 	 if (DO_SPEC)
 	    STRIDE_4UB(spec, start * spec_stride);
 	 if (DO_FOG)
-	    STRIDE_F(fog, start * fog_stride);
+	    /*STRIDE_F(fog, start * fog_stride);*/
+	    fog =  (GLfloat (*)[4])((GLfloat *)fog + start * fog_stride);
       }
 
       for (i=start; i < end; i++, v = (VERTEX *)((GLubyte *)v + stride)) {
@@ -265,8 +269,9 @@ static void TAG(emit)( GLcontext *ctx,
 	    STRIDE_4UB(spec, spec_stride);
 	 }
 	 if (DO_FOG) {
-	    v->v.specular.alpha = fog[0] * 255.0;
-	    STRIDE_F(fog, fog_stride);
+	    v->v.specular.alpha = fog[0][0] * 255.0;
+	    /*STRIDE_F(fog, fog_stride);*/
+	    fog =  (GLfloat (*)[4])((GLfloat *)fog +  fog_stride);
 	 }
 	 if (DO_TEX0) {
 	    v->v.u0 = tc0[0][0];
@@ -364,7 +369,7 @@ static void TAG(emit)( GLcontext *ctx,
 	    v->v.specular.blue  = spec[i][2];
 	 }
 	 if (DO_FOG) {
-	    v->v.specular.alpha = fog[i] * 255.0;
+	    v->v.specular.alpha = fog[i][0] * 255.0;
 	 }
 	 if (DO_TEX0) {
 	    v->v.u0 = tc0[i][0];
@@ -417,8 +422,8 @@ static void TAG(emit)( GLcontext *ctx, GLuint start, GLuint end,
    struct vertex_buffer *VB = &TNL_CONTEXT(ctx)->vb;
    GLubyte (*col)[4];
    GLuint col_stride;
-   GLfloat (*coord)[4] = VB->ProjectedClipPtr->data;
-   GLuint coord_stride = VB->ProjectedClipPtr->stride;
+   GLfloat (*coord)[4] = VB->NdcPtr->data;
+   GLuint coord_stride = VB->NdcPtr->stride;
    GLfloat *v = (GLfloat *)dest;
    const GLubyte *mask = VB->ClipMask;
    const GLfloat *s = GET_VIEWPORT_MAT();
@@ -459,11 +464,11 @@ static void TAG(emit)( GLcontext *ctx, GLuint start, GLuint end,
 	       *(GLuint *)&v[3] = LE32_TO_CPU(*(GLuint *)col);
 	    }
 	    else {
-	       GLubyte *b = (GLubyte *)&v[3];
-	       b[0] = col[0][2];
-	       b[1] = col[0][1];
-	       b[2] = col[0][0];
-	       b[3] = col[0][3];
+	       VERTEX_COLOR *c = (VERTEX_COLOR *)&v[3];
+	       c->blue  = col[0][2];
+	       c->green = col[0][1];
+	       c->red   = col[0][0];
+	       c->alpha = col[0][3];
 	    }
 	    STRIDE_4UB( col, col_stride );
 	 }
@@ -483,11 +488,11 @@ static void TAG(emit)( GLcontext *ctx, GLuint start, GLuint end,
 	       *(GLuint *)&v[3] = LE32_TO_CPU(*(GLuint *)&col[i]);
 	    }
 	    else {
-	       GLubyte *b = (GLubyte *)&v[3];
-	       b[0] = col[i][2];
-	       b[1] = col[i][1];
-	       b[2] = col[i][0];
-	       b[3] = col[i][3];
+	       VERTEX_COLOR *c = (VERTEX_COLOR *)&v[3];
+	       c->blue  = col[i][2];
+	       c->green = col[i][1];
+	       c->red   = col[i][0];
+	       c->alpha = col[i][3];
 	    }
 	 }
 /*  	 fprintf(stderr, "vert %d: %.2f %.2f %.2f %x\n",  */
@@ -528,11 +533,11 @@ static void TAG(emit)( GLcontext *ctx, GLuint start, GLuint end,
 	 *(GLuint *)v = LE32_TO_CPU(*(GLuint *)col[0]);
       }
       else {
-	 GLubyte *b = (GLubyte *)v;
-	 b[0] = col[0][2];
-	 b[1] = col[0][1];
-	 b[2] = col[0][0];
-	 b[3] = col[0][3];
+	 VERTEX_COLOR *c = (VERTEX_COLOR *)v;
+	 c->blue  = col[0][2];
+	 c->green = col[0][1];
+	 c->red   = col[0][0];
+	 c->alpha = col[0][3];
       }
       STRIDE_4UB( col, col_stride );
    }
@@ -671,8 +676,8 @@ static void TAG(interp)( GLcontext *ctx,
 	       INTERP_F( t, dst->pv.v0, out->pv.v0, in->pv.v0 );
 	       INTERP_F( t, dst->pv.q0, out->pv.q0, in->pv.q0 );
 	    } else {
-	       GLfloat wout = VB->ProjectedClipPtr->data[eout][3];
-	       GLfloat win = VB->ProjectedClipPtr->data[ein][3];
+	       GLfloat wout = VB->NdcPtr->data[eout][3];
+	       GLfloat win = VB->NdcPtr->data[ein][3];
 	       GLfloat qout = out->pv.w / wout;
 	       GLfloat qin = in->pv.w / win;
 	       GLfloat qdst, rqdst;
@@ -705,7 +710,7 @@ static void TAG(interp)( GLcontext *ctx,
 	 }
       }
       else if (DO_PTEX) {
-	 dst->pv.q0 = 0.0;	/* must be a valid float on radeon */
+	 dst->pv.q1 = 0.0;	/* must be a valid float on radeon */
       }
       if (DO_TEX2) {
 	 if (DO_PTEX) {
