@@ -1,8 +1,8 @@
-/* $Id: vtxfmt.c,v 1.6.2.1 2002/04/09 12:13:08 keithw Exp $ */
+/* $Id: vtxfmt.c,v 1.13 2002/10/24 23:57:21 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.1
  *
  * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
  *
@@ -31,6 +31,7 @@
 #include "glheader.h"
 #include "api_loopback.h"
 #include "context.h"
+#include "imports.h"
 #include "mtypes.h"
 #include "state.h"
 #include "vtxfmt.h"
@@ -54,18 +55,16 @@
    ASSERT( tnl->Current );						\
    ASSERT( tnl->SwapCount < NUM_VERTEX_FORMAT_ENTRIES );		\
 									\
-   /* Save the swapped function's dispatch entry so it can be		\
-    * restored later.							\
-    */									\
+   /* Save the swapped function's dispatch entry so it can be */	\
+   /* restored later. */						\
    tnl->Swapped[tnl->SwapCount][0] = (void *)&(ctx->Exec->FUNC);	\
    tnl->Swapped[tnl->SwapCount][1] = (void *)TAG(FUNC);			\
    tnl->SwapCount++;							\
 									\
    if ( 0 )								\
-      fprintf( stderr, "   swapping gl" #FUNC"...\n" );			\
+      _mesa_debug(ctx, "   swapping gl" #FUNC"...\n" );			\
 									\
-   /* Install the tnl function pointer.					\
-    */									\
+   /* Install the tnl function pointer.	*/				\
    ctx->Exec->FUNC = tnl->Current->FUNC;				\
 }
 
@@ -126,18 +125,17 @@ static void install_vtxfmt( struct _glapi_table *tab, GLvertexformat *vfmt )
    tab->Vertex3fv = vfmt->Vertex3fv;
    tab->Vertex4f = vfmt->Vertex4f;
    tab->Vertex4fv = vfmt->Vertex4fv;
+   tab->CallList = vfmt->CallList;
    tab->Begin = vfmt->Begin;
    tab->End = vfmt->End;
-
-/*     tab->NewList = vfmt->NewList; */
-   tab->CallList = vfmt->CallList;
-
+   tab->VertexAttrib4fNV = vfmt->VertexAttrib4fNV;
    tab->Rectf = vfmt->Rectf;
    tab->DrawArrays = vfmt->DrawArrays;
    tab->DrawElements = vfmt->DrawElements;
    tab->DrawRangeElements = vfmt->DrawRangeElements;
    tab->EvalMesh1 = vfmt->EvalMesh1;
    tab->EvalMesh2 = vfmt->EvalMesh2;
+   assert(tab->EvalMesh2);
 }
 
 
