@@ -1,10 +1,10 @@
-/* $Id: vtxfmt_tmp.h,v 1.4.2.1 2001/11/26 23:45:36 brianp Exp $ */
+/* $Id: vtxfmt_tmp.h,v 1.10 2002/10/08 23:58:55 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  4.0.1
+ * Version:  4.1
  *
- * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2002  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,109 +34,109 @@
 static void TAG(ArrayElement)( GLint i )
 {
    PRE_LOOPBACK( ArrayElement );
-   glArrayElement( i );
+   _glapi_Dispatch->ArrayElement( i );
 }
 
-static void TAG(Color3f)( GLfloat a, GLfloat b, GLfloat c )
+static void TAG(Color3f)( GLfloat r, GLfloat g, GLfloat b )
 {
    PRE_LOOPBACK( Color3f );
-   glColor3f( a, b, c );
+   _glapi_Dispatch->Color3f( r, g, b );
 }
 
 static void TAG(Color3fv)( const GLfloat *v )
 {
    PRE_LOOPBACK( Color3fv );
-   glColor3fv( v );
+   _glapi_Dispatch->Color3fv( v );
 }
 
-static void TAG(Color3ub)( GLubyte a, GLubyte b, GLubyte c )
+static void TAG(Color3ub)( GLubyte r, GLubyte g, GLubyte b )
 {
    PRE_LOOPBACK( Color3ub );
-   glColor3ub( a, b, c );
+   _glapi_Dispatch->Color3ub( r, g, b );
 }
 
 static void TAG(Color3ubv)( const GLubyte *v )
 {
    PRE_LOOPBACK( Color3ubv );
-   glColor3ubv( v );
+   _glapi_Dispatch->Color3ubv( v );
 }
 
-static void TAG(Color4f)( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
+static void TAG(Color4f)( GLfloat r, GLfloat g, GLfloat b, GLfloat a )
 {
    PRE_LOOPBACK( Color4f );
-   glColor4f( a, b, c, d );
+   _glapi_Dispatch->Color4f( r, g, b, a );
 }
 
 static void TAG(Color4fv)( const GLfloat *v )
 {
    PRE_LOOPBACK( Color4fv );
-   glColor4fv( v );
+   _glapi_Dispatch->Color4fv( v );
 }
 
-static void TAG(Color4ub)( GLubyte a, GLubyte b, GLubyte c, GLubyte d )
+static void TAG(Color4ub)( GLubyte r, GLubyte g, GLubyte b, GLubyte a )
 {
    PRE_LOOPBACK( Color4ub );
-   glColor4ub( a, b, c, d );
+   _glapi_Dispatch->Color4ub( r, g, b, a );
 }
 
 static void TAG(Color4ubv)( const GLubyte *v )
 {
    PRE_LOOPBACK( Color4ubv );
-   glColor4ubv( v );
+   _glapi_Dispatch->Color4ubv( v );
 }
 
-static void TAG(EdgeFlag)( GLboolean a )
+static void TAG(EdgeFlag)( GLboolean e )
 {
    PRE_LOOPBACK( EdgeFlag );
-   glEdgeFlag( a );
+   _glapi_Dispatch->EdgeFlag( e );
 }
 
 static void TAG(EdgeFlagv)( const GLboolean *v )
 {
    PRE_LOOPBACK( EdgeFlagv );
-   glEdgeFlagv( v );
+   _glapi_Dispatch->EdgeFlagv( v );
 }
 
-static void TAG(EvalCoord1f)( GLfloat a )
+static void TAG(EvalCoord1f)( GLfloat s )
 {
    PRE_LOOPBACK( EvalCoord1f );
-   glEvalCoord1f( a );
+   _glapi_Dispatch->EvalCoord1f( s );
 }
 
 static void TAG(EvalCoord1fv)( const GLfloat *v )
 {
    PRE_LOOPBACK( EvalCoord1fv );
-   glEvalCoord1fv( v );
+   _glapi_Dispatch->EvalCoord1fv( v );
 }
 
-static void TAG(EvalCoord2f)( GLfloat a, GLfloat b )
+static void TAG(EvalCoord2f)( GLfloat s, GLfloat t )
 {
    PRE_LOOPBACK( EvalCoord2f );
-   glEvalCoord2f( a, b );
+   _glapi_Dispatch->EvalCoord2f( s, t );
 }
 
 static void TAG(EvalCoord2fv)( const GLfloat *v )
 {
    PRE_LOOPBACK( EvalCoord2fv );
-   glEvalCoord2fv( v );
+   _glapi_Dispatch->EvalCoord2fv( v );
 }
 
-static void TAG(EvalPoint1)( GLint a )
+static void TAG(EvalPoint1)( GLint i )
 {
    PRE_LOOPBACK( EvalPoint1 );
-   glEvalPoint1( a );
+   _glapi_Dispatch->EvalPoint1( i );
 }
 
-static void TAG(EvalPoint2)( GLint a, GLint b )
+static void TAG(EvalPoint2)( GLint i, GLint j )
 {
    PRE_LOOPBACK( EvalPoint2 );
-   glEvalPoint2( a, b );
+   _glapi_Dispatch->EvalPoint2( i, j );
 }
 
-static void TAG(FogCoordfEXT)( GLfloat a )
+static void TAG(FogCoordfEXT)( GLfloat f )
 {
    PRE_LOOPBACK( FogCoordfEXT );
-   _glapi_Dispatch->FogCoordfEXT( a );
+   _glapi_Dispatch->FogCoordfEXT( f );
 }
 
 static void TAG(FogCoordfvEXT)( const GLfloat *v )
@@ -145,90 +145,90 @@ static void TAG(FogCoordfvEXT)( const GLfloat *v )
    _glapi_Dispatch->FogCoordfvEXT( v );
 }
 
-static void TAG(Indexi)( GLint a )
+static void TAG(Indexi)( GLint i )
 {
    PRE_LOOPBACK( Indexi );
-   glIndexi( a );
+   _glapi_Dispatch->Indexi( i );
 }
 
 static void TAG(Indexiv)( const GLint *v )
 {
    PRE_LOOPBACK( Indexiv );
-   glIndexiv( v );
+   _glapi_Dispatch->Indexiv( v );
 }
 
 static void TAG(Materialfv)( GLenum face, GLenum pname, const GLfloat *v )
 {
    PRE_LOOPBACK( Materialfv );
-   glMaterialfv( face, pname, v );
+   _glapi_Dispatch->Materialfv( face, pname, v );
 }
 
 static void TAG(MultiTexCoord1fARB)( GLenum target, GLfloat a )
 {
    PRE_LOOPBACK( MultiTexCoord1fARB );
-   glMultiTexCoord1fARB( target, a );
+   _glapi_Dispatch->MultiTexCoord1fARB( target, a );
 }
 
 static void TAG(MultiTexCoord1fvARB)( GLenum target, const GLfloat *tc )
 {
    PRE_LOOPBACK( MultiTexCoord1fvARB );
-   glMultiTexCoord1fvARB( target, tc );
+   _glapi_Dispatch->MultiTexCoord1fvARB( target, tc );
 }
 
-static void TAG(MultiTexCoord2fARB)( GLenum target, GLfloat a, GLfloat b )
+static void TAG(MultiTexCoord2fARB)( GLenum target, GLfloat s, GLfloat t )
 {
    PRE_LOOPBACK( MultiTexCoord2fARB );
-   glMultiTexCoord2fARB( target, a, b );
+   _glapi_Dispatch->MultiTexCoord2fARB( target, s, t );
 }
 
 static void TAG(MultiTexCoord2fvARB)( GLenum target, const GLfloat *tc )
 {
    PRE_LOOPBACK( MultiTexCoord2fvARB );
-   glMultiTexCoord2fvARB( target, tc );
+   _glapi_Dispatch->MultiTexCoord2fvARB( target, tc );
 }
 
-static void TAG(MultiTexCoord3fARB)( GLenum target, GLfloat a,
-				     GLfloat b, GLfloat c )
+static void TAG(MultiTexCoord3fARB)( GLenum target, GLfloat s,
+				     GLfloat t, GLfloat r )
 {
    PRE_LOOPBACK( MultiTexCoord3fARB );
-   glMultiTexCoord3fARB( target, a, b, c );
+   _glapi_Dispatch->MultiTexCoord3fARB( target, s, t, r );
 }
 
 static void TAG(MultiTexCoord3fvARB)( GLenum target, const GLfloat *tc )
 {
    PRE_LOOPBACK( MultiTexCoord3fvARB );
-   glMultiTexCoord3fvARB( target, tc );
+   _glapi_Dispatch->MultiTexCoord3fvARB( target, tc );
 }
 
-static void TAG(MultiTexCoord4fARB)( GLenum target, GLfloat a,
-				     GLfloat b, GLfloat c, GLfloat d )
+static void TAG(MultiTexCoord4fARB)( GLenum target, GLfloat s,
+				     GLfloat t, GLfloat r, GLfloat q )
 {
    PRE_LOOPBACK( MultiTexCoord4fARB );
-   glMultiTexCoord4fARB( target, a, b, c, d );
+   _glapi_Dispatch->MultiTexCoord4fARB( target, s, t, r, q );
 }
 
 static void TAG(MultiTexCoord4fvARB)( GLenum target, const GLfloat *tc )
 {
    PRE_LOOPBACK( MultiTexCoord4fvARB );
-   glMultiTexCoord4fvARB( target, tc );
+   _glapi_Dispatch->MultiTexCoord4fvARB( target, tc );
 }
 
-static void TAG(Normal3f)( GLfloat a, GLfloat b, GLfloat c )
+static void TAG(Normal3f)( GLfloat x, GLfloat y, GLfloat z )
 {
    PRE_LOOPBACK( Normal3f );
-   glNormal3f( a, b, c );
+   _glapi_Dispatch->Normal3f( x, y, z );
 }
 
 static void TAG(Normal3fv)( const GLfloat *v )
 {
    PRE_LOOPBACK( Normal3fv );
-   glNormal3fv( v );
+   _glapi_Dispatch->Normal3fv( v );
 }
 
-static void TAG(SecondaryColor3fEXT)( GLfloat a, GLfloat b, GLfloat c )
+static void TAG(SecondaryColor3fEXT)( GLfloat r, GLfloat g, GLfloat b )
 {
    PRE_LOOPBACK( SecondaryColor3fEXT );
-   _glapi_Dispatch->SecondaryColor3fEXT( a, b, c );
+   _glapi_Dispatch->SecondaryColor3fEXT( r, g, b );
 }
 
 static void TAG(SecondaryColor3fvEXT)( const GLfloat *v )
@@ -237,10 +237,10 @@ static void TAG(SecondaryColor3fvEXT)( const GLfloat *v )
    _glapi_Dispatch->SecondaryColor3fvEXT( v );
 }
 
-static void TAG(SecondaryColor3ubEXT)( GLubyte a, GLubyte b, GLubyte c )
+static void TAG(SecondaryColor3ubEXT)( GLubyte r, GLubyte g, GLubyte b )
 {
    PRE_LOOPBACK( SecondaryColor3ubEXT );
-   _glapi_Dispatch->SecondaryColor3ubEXT( a, b, c );
+   _glapi_Dispatch->SecondaryColor3ubEXT( r, g, b );
 }
 
 static void TAG(SecondaryColor3ubvEXT)( const GLubyte *v )
@@ -249,125 +249,125 @@ static void TAG(SecondaryColor3ubvEXT)( const GLubyte *v )
    _glapi_Dispatch->SecondaryColor3ubvEXT( v );
 }
 
-static void TAG(TexCoord1f)( GLfloat a )
+static void TAG(TexCoord1f)( GLfloat s )
 {
    PRE_LOOPBACK( TexCoord1f );
-   glTexCoord1f( a );
+   _glapi_Dispatch->TexCoord1f( s );
 }
 
 static void TAG(TexCoord1fv)( const GLfloat *tc )
 {
    PRE_LOOPBACK( TexCoord1fv );
-   glTexCoord1fv( tc );
+   _glapi_Dispatch->TexCoord1fv( tc );
 }
 
-static void TAG(TexCoord2f)( GLfloat a, GLfloat b )
+static void TAG(TexCoord2f)( GLfloat s, GLfloat t )
 {
    PRE_LOOPBACK( TexCoord2f );
-   glTexCoord2f( a, b );
+   _glapi_Dispatch->TexCoord2f( s, t );
 }
 
 static void TAG(TexCoord2fv)( const GLfloat *tc )
 {
    PRE_LOOPBACK( TexCoord2fv );
-   glTexCoord2fv( tc );
+   _glapi_Dispatch->TexCoord2fv( tc );
 }
 
-static void TAG(TexCoord3f)( GLfloat a, GLfloat b, GLfloat c )
+static void TAG(TexCoord3f)( GLfloat s, GLfloat t, GLfloat r )
 {
    PRE_LOOPBACK( TexCoord3f );
-   glTexCoord3f( a, b, c );
+   _glapi_Dispatch->TexCoord3f( s, t, r );
 }
 
 static void TAG(TexCoord3fv)( const GLfloat *tc )
 {
    PRE_LOOPBACK( TexCoord3fv );
-   glTexCoord3fv( tc );
+   _glapi_Dispatch->TexCoord3fv( tc );
 }
 
-static void TAG(TexCoord4f)( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
+static void TAG(TexCoord4f)( GLfloat s, GLfloat t, GLfloat r, GLfloat q )
 {
    PRE_LOOPBACK( TexCoord4f );
-   glTexCoord4f( a, b, c, d );
+   _glapi_Dispatch->TexCoord4f( s, t, r, q );
 }
 
 static void TAG(TexCoord4fv)( const GLfloat *tc )
 {
    PRE_LOOPBACK( TexCoord4fv );
-   glTexCoord4fv( tc );
+   _glapi_Dispatch->TexCoord4fv( tc );
 }
 
-static void TAG(Vertex2f)( GLfloat a, GLfloat b )
+static void TAG(Vertex2f)( GLfloat x, GLfloat y )
 {
    PRE_LOOPBACK( Vertex2f );
-   glVertex2f( a, b );
+   _glapi_Dispatch->Vertex2f( x, y );
 }
 
-static void TAG(Vertex2fv)( const GLfloat *obj )
+static void TAG(Vertex2fv)( const GLfloat *v )
 {
    PRE_LOOPBACK( Vertex2fv );
-   glVertex2fv( obj );
+   _glapi_Dispatch->Vertex2fv( v );
 }
 
-static void TAG(Vertex3f)( GLfloat a, GLfloat b, GLfloat c )
+static void TAG(Vertex3f)( GLfloat x, GLfloat y, GLfloat z )
 {
    PRE_LOOPBACK( Vertex3f );
-   glVertex3f( a, b, c );
+   _glapi_Dispatch->Vertex3f( x, y, z );
 }
 
-static void TAG(Vertex3fv)( const GLfloat *obj )
+static void TAG(Vertex3fv)( const GLfloat *v )
 {
    PRE_LOOPBACK( Vertex3fv );
-   glVertex3fv( obj );
+   _glapi_Dispatch->Vertex3fv( v );
 }
 
-static void TAG(Vertex4f)( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
+static void TAG(Vertex4f)( GLfloat x, GLfloat y, GLfloat z, GLfloat w )
 {
    PRE_LOOPBACK( Vertex4f );
-   glVertex4f( a, b, c, d );
+   _glapi_Dispatch->Vertex4f( x, y, z, w );
 }
 
-static void TAG(Vertex4fv)( const GLfloat *obj )
+static void TAG(Vertex4fv)( const GLfloat *v )
 {
    PRE_LOOPBACK( Vertex4fv );
-   glVertex4fv( obj );
+   _glapi_Dispatch->Vertex4fv( v );
 }
 
 static void TAG(CallList)( GLuint i )
 {
    PRE_LOOPBACK( CallList );
-   glCallList( i );
+   _glapi_Dispatch->CallList( i );
 }
 
 static void TAG(Begin)( GLenum mode )
 {
    PRE_LOOPBACK( Begin );
-   glBegin( mode );
+   _glapi_Dispatch->Begin( mode );
 }
 
 static void TAG(End)( void )
 {
    PRE_LOOPBACK( End );
-   glEnd();
+   _glapi_Dispatch->End();
 }
 
 static void TAG(Rectf)( GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2 )
 {
    PRE_LOOPBACK( Rectf );
-   glRectf( x1, y1, x2, y2 );
+   _glapi_Dispatch->Rectf( x1, y1, x2, y2 );
 }
 
 static void TAG(DrawArrays)( GLenum mode, GLint start, GLsizei count )
 {
    PRE_LOOPBACK( DrawArrays );
-   glDrawArrays( mode, start, count );
+   _glapi_Dispatch->DrawArrays( mode, start, count );
 }
 
 static void TAG(DrawElements)( GLenum mode, GLsizei count, GLenum type,
 			       const void *indices )
 {
    PRE_LOOPBACK( DrawElements );
-   glDrawElements( mode, count, type, indices );
+   _glapi_Dispatch->DrawElements( mode, count, type, indices );
 }
 
 static void TAG(DrawRangeElements)( GLenum mode, GLuint start,
@@ -375,20 +375,32 @@ static void TAG(DrawRangeElements)( GLenum mode, GLuint start,
 				    GLenum type, const void *indices )
 {
    PRE_LOOPBACK( DrawRangeElements );
-   glDrawRangeElements( mode, start, end, count, type, indices );
+   _glapi_Dispatch->DrawRangeElements( mode, start, end, count, type, indices );
 }
 
 static void TAG(EvalMesh1)( GLenum mode, GLint i1, GLint i2 )
 {
    PRE_LOOPBACK( EvalMesh1 );
-   glEvalMesh1( mode, i1, i2 );
+   _glapi_Dispatch->EvalMesh1( mode, i1, i2 );
 }
 
 static void TAG(EvalMesh2)( GLenum mode, GLint i1, GLint i2,
 			    GLint j1, GLint j2 )
 {
    PRE_LOOPBACK( EvalMesh2 );
-   glEvalMesh2( mode, i1, i2, j1, j2 );
+   _glapi_Dispatch->EvalMesh2( mode, i1, i2, j1, j2 );
+}
+
+static void TAG(VertexAttrib4fNV)( GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w )
+{
+   PRE_LOOPBACK( VertexAttrib4fNV );
+   _glapi_Dispatch->VertexAttrib4fNV( index, x, y, z, w );
+}
+
+static void TAG(VertexAttrib4fvNV)( GLuint index, const GLfloat *v )
+{
+   PRE_LOOPBACK( VertexAttrib4fNV );
+   _glapi_Dispatch->VertexAttrib4fvNV( index, v );
 }
 
 
@@ -446,12 +458,14 @@ static GLvertexformat TAG(vtxfmt) = {
    TAG(CallList),
    TAG(Begin),
    TAG(End),
+   TAG(VertexAttrib4fNV),
+   TAG(VertexAttrib4fvNV),
    TAG(Rectf),
    TAG(DrawArrays),
    TAG(DrawElements),
    TAG(DrawRangeElements),
    TAG(EvalMesh1),
-   TAG(EvalMesh2),
+   TAG(EvalMesh2)
 };
 
 #undef TAG
